#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import Tkinter as tk
import tkMessageBox as tmb
import Pmw, tkFont, ImageTk
import DebugTools
import threading, Queue, time, sys, platform
from CADD.Raccoon2 import HelperFunctionsN3P as hf



class RaccoonDefaultWidget:
    """ base widget for defining all the common aestetic settings"""
    def __init__(self, parent=None, iconpath=None):
        if not parent == None:
            self.parent = parent
        else:
            self.parent = tk.Tk()

        if iconpath:
            self.iconpath = iconpath


        self.sysarch = platform.uname()[0]
        # BUG
        # this is a fix that's needed to parse multiple files selections
        # due to a bug in askopenfilenames ( http://bugs.python.org/issue5712 )
        t = tk.Tk()
        t.withdraw()
        self._listfixer = t.splitlist
        #self._listfixer = self.parent.splitlist
        # BUG
        if self.sysarch=='Windows':
            normsize = 8
            smallsize = 7
        else:
            normsize = 9
            smallsize = 8
        family = 'Arial'
        #family = 'Helvetica'
        self.FONT = tkFont.Font(family=family,size=normsize)
        self.FONTbold = tkFont.Font(family=family,size=normsize,weight="bold")
        self.FONTsmall = tkFont.Font(family=family,size=smallsize)
        if self.parent:
            self.parent.option_add( "*font", "Arial %s bold" % normsize)
            self.parent.option_add( "*font", "Arial %s" % normsize) 
        #self.parent.option_add('*Background', '#969b9d')
        self.BORDER = { 'bd':1,'highlightbackground':'black',
                    'borderwidth':2,'highlightcolor':'black','highlightthickness':1}

class TabBase(DebugTools.DebugObj):
    def __init__(self, app, parent, debug = False):
        """ app         is the calling application
            parent      is the container where widgets will be created (
        """
        DebugTools.DebugObj.__init__(self, debug)
        self.app = app
        self.parent = parent
        self.frame = tk.Frame(parent)

    def resetFrame(self):
        self.frame.pack_forget()
        self.frame = tk.Frame(self.parent)



class PanedManager(RaccoonDefaultWidget):
    """
    """
    def __init__(self, app=None, parent=None, wtitle='PanedManager', ltitle='Left', rtitle='Right',
        lwidth=0, rwidth=0):
        RaccoonDefaultWidget.__init__(self, parent)
        self.app = app
        self.wtitle = wtitle
        self.ltitle = ltitle
        self.rtitle = rtitle
        self.lwidth = lwidth
        self.rwidth = rwidth

        self.initIcons()
        self.makeWin()


    def __call__(self): #, geom='centerscreenalways'):
        self.win.activate() #geometry=geom)

    def destroy(self, event=None):
        """
        """
        # trigger the event refresh of server list
        self.win.deactivate()

    def makeWin(self):
        """ """
        self.win = Pmw.Dialog(self.parent, title=self.wtitle, 
            buttons = ('Close',), command = self.destroy)

        bbox = self.win.component('buttonbox')
        for i in range(bbox.numbuttons()):
            bbox.button(i).configure(font=self.FONT, default='disabled', **self.BORDER)
            #bbox.button(i).configure(relief='raised')

        self.pane = Pmw.PanedWidget(self.win.interior(), orient='horizontal', handlesize=-1,
            separatorthickness=10, separatorrelief='raised', )


        left = self.pane.add('info', min=self.lwidth, size=self.lwidth)
        right =  self.pane.add('viewer', min=self.rwidth, size=self.rwidth)
        handle = self.pane.component('handle-1')
        sep = self.pane.component('separator-1')

        self.pane.component('handle-1').place_forget()
        self.pane.component('handle-1').forget()
        self.pane.component('handle-1').pack_forget()
        self.pane.component('handle-1').grid_forget()
        self.pane.component('separator-1').configure(bd =2, #bg = '#999999'
            highlightthickness=1, highlightbackground='black', highlightcolor='black')
        lgroup = Pmw.Group(left, tag_text=self.ltitle, tag_font=self.FONTbold)
        lgroup.pack(expand=1,fill='both',side='top', anchor='center', padx=5,pady=5)
        self.left = lgroup.interior()

        rgroup = Pmw.Group(right, tag_text=self.rtitle, tag_font=self.FONTbold)
        rgroup.pack(expand=1,fill='both',side='left', anchor='center', padx=5,pady=5)
        self.right = rgroup.interior()

        # separator for rearranging 
        spacer = tk.Frame(self.right, width=6)
        spacer.pack(expand=0, fill='y',side='left', anchor='w')


        # nail handle
        tk.Frame(sep,height=40,width=4,bg='#fffeee',relief='sunken',bd=1,highlightbackground='black',
            highlightthickness=1).pack( anchor='center', padx=2,pady=2,side='left',expand=0,fill=None)

        # LEFT
        self.listcontainer = Pmw.ScrolledListBox(parent = self.left)
        self.listcontainer.component('listbox').configure(bg='white', font=self.FONT)
        self.listcontainer.pack(expand=1, fill='both')

        # RIGHT
        # pack everything
        self.pane.pack(expand=1,fill='both')

        self.pane.component('hull').configure(width=800, height=400)
    



class RacMenu(RaccoonDefaultWidget):
    """ Provide a self-destructible one-level pop-up menu
        (similar to what's usually associated to right-button 
        mouse clicks) and save expensive time-machine
        bills for getting back to the '80 to get 
        'Dismiss' buttons supplies.

        parent  : the parent widget (usually a button)
        items   : a nested list

             items = [ ['title'],                      <- only one entry : title (separator automatically added)
                                                                           [optional, leave empty entry for nothing]
        
                       ['entry', 'status', callback],  <- thre entries   : menu entries (text, normal/disabled, function)
                       [],                             <- empty          : separator
                       ['entry', 'status', callback],  <- thre entries   : menu entries (text, normal/disabled, function)
                       ['entry', 'status', callback],  <- thre entries   : menu entries (text, normal/disabled, function)
                     ]
        toolbar : optional, is usually a frame that contain the parent widget and 
                  other widgets; it is used to keep track of multiple menus in the same tooblar
        placement: close, under, tuple(x,y)... (others don't work yet)
        floating : it is not tied to a specific widget (i.e. button toolbar)

    """
    def __init__(self, parent, items=[], toolbar=False, placement='close', floating=False):
        RaccoonDefaultWidget.__init__(self, parent)
        self.menu = tk.Menu(parent, tearoff=False, takefocus=1)
        if not hasattr(self.parent, 'menu'):
            self.parent.menu = None
        self.items = items
        self._count = 0
        self._maxlen = 0
        self._posted = False
        self._caller = None
        self.floating = floating
        if toolbar:
            self.toolbar = toolbar
            self.toolbar.menu = None
        else:
            self.toolbar = False
        self.placement = placement
        self.parent.configure(relief='raised')
        self.populate()

    def populate(self):
        """ parse items and fill the menu """
        self.parent.configure(relief='raised')
        for i in range(len(self.items)):
            entry = self.items[i]
            if i == 0:              # LABEL
                if not entry == None:
                    self._count += 2
                    self.menu.add_command(label=entry[0], state='disable', 
                            font=self.FONTbold, foreground='white', background='black' )
                    self._maxlen = max( self._maxlen, len(entry[0]))
                    self.menu.add_separator()
            else:
                if len(entry) == 0: # SEPARATOR
                    self._count += 1
                    self.menu.add_separator()
                else:               # MENU ENTRY
                    self._count += 1
                    self._maxlen = max( self._maxlen, len(entry[0]))
                    l, s, c = entry
                    self.menu.add_command(label = ' '+ l +'  ',
                                          state= s,
                                          command = c,
                                          font = self.FONT)
        self.parent.configure(relief='raised')
        if not self.floating:
            self.parent.bind('<Button-1>', self)
            self.parent.bind('<Leave>', self._gozer)
            self.parent.bind('<FocusOut>', self._gozer)
        self.menu.configure(disabledforeground='white')
        self.menu.bind('<Leave>', self._gozer)
        self.menu.bind('<FocusOut>', self._gozer)
                
    def __call__(self, event=None):
        """ post the menu visible
        
            REMEMBER that all arrangements must not have discontinuities between
            the posted menu and the parent, otherwise a '<leave>' event would
            be triggered, calling _gozer)
        """
        self.parent.configure(relief='raised')
        if self._posted:
            self._gozer(force=True)
        if self.toolbar:
            # check if the toolbar doesn't have other
            # menu's posted
            if hasattr(self.toolbar, 'menu'):
                if self.toolbar.menu:
                    self.toolbar.menu._gozer(force=True)
        if self.placement == 'close':
            # NOTE the placement do not allow gaps! (otherwise they
            #      will trigger the <leave> event!)
            x = self.parent.winfo_rootx() + self.parent.winfo_width()# +1
            y = self.parent.winfo_rooty()

            self.menu.post(x, y)
            self.menu.bind('<Leave>', self._gozer)
            self._posted = True
            self.parent.menu = self
            if self.toolbar:
                self.toolbar.menu = self
            return

        elif self.placement == 'under':
            x = self.parent.winfo_rootx() #+ self.parent.winfo_width()# +1
            y = self.parent.winfo_rooty() + self.parent.winfo_height()
            self.menu.post(x, y)
            self.menu.bind('<Leave>', self._gozer)
            self._posted = True
            self.parent.menu = self
            if self.toolbar:
                self.toolbar.menu = self
            return            


        elif isinstance(self.placement, tuple):
            x, y = self.placement
            self.menu.post(x, y)
            self.menu.bind('<Leave>', self._gozer)
            self._posted = True
            self.parent.menu = self
            if self.toolbar:
                self.toolbar.menu = self
            self.menu.bind('<Leave>', self._gozer)
            self.menu.bind('<FocusOut>', self._gozer)
            return

        elif self.placement == 'left':
            x_off = -self._maxlen * 10
            y_off = -8
        elif self.placement == 'center':
            x_off = -self._maxlen * 5 # offset pixels per char
            y_off = -self.menu.yposition( self._count / 2)
        self._posted = True
        self.menu.post(event.x_root+x_off, event.y_root+y_off)


    def _gozer(self, event=None, force=False):
        """the Gozerian, the Destructor"""
        self.parent.configure(relief='raised')
        if force:
            self.menu.unpost()
            self._posted = False
            self.parent.menu = None
            return
        if not self._posted: return
        ex, ey = event.x_root, event.y_root

        mx0, my0 = self.menu.winfo_rootx(), self.menu.winfo_rooty()
        mw, mh = self.menu.winfo_width(), self.menu.winfo_height()
        mx1, my1 = mx0+mw, my0+mh

        px0, py0 = self.parent.winfo_rootx(), self.parent.winfo_rooty()
        pw, ph = self.parent.winfo_width(), self.parent.winfo_height()
        px1, py1 = px0+pw, py0+ph
        
        in_menu = False
        if mx0 <= ex < mx1:
            if my0 <= ey < my1:
                in_menu = True
        in_butt = False
        if not self.floating:
            if px0 <= ex < px1 :
                if py0 <= ey <= py1:
                    in_butt = True
        if in_butt or in_menu:
            return
        self._posted = False
        self.parent.menu = None
        if self.toolbar:
            self.toolbar.menu = None
        self.menu.unpost()

    def _gozer__BETTER_BUT_OFF(self, event=None, force=True):
        """the Gozerian, the Destructor"""
        # mouse coords
        x, y = event.x_root, event.y_root

        # calling button coords
        bx, by = self.parent.winfo_rootx(), self.parent.winfo_rooty()
        bw, bh = self.parent.winfo_width(), self.parent.winfo_width()

        # menu button coords
        mx, my = self.menu.winfo_rootx(), self.menu.winfo_rooty()
        mw, mh = self.menu.winfo_width(), self.menu.winfo_height()

        in_menu = ((x>mx)and(x<mx+mw)) and ((y>my)and(y< my+mh))
        in_button = ((x>bx)and(x<bx+bw)) and ((y>by)and(y< by+bh))
        print "\n\n\nIN MENU", in_menu
        print "IN BUTTON", in_button
        if in_menu:
            print "INMENU< RETURNING"
            return
        if in_button:
            print "INBUTTION< RETURNONG"
            return
        self._posted = False
        self.menu.unpost()



        """    
            


        if event.widget.winfo_containing(event.x, event.y) == self.menu:
            return
        event = None
        if event: 
            #print "EVENT", event,
            event.widget.unpost()
            self._posted = False
        else:
            self._posted = False
            self.menu.unpost()
        """

    """
    NOT ENOUGH TIME TO EXPERIMENT
    def __call__(self, event=None):
        "" post the menu visible""
        if self._posted:
            return
        #center = self.menu.yposition( self._count / 2)

        # use .configure(postcommand = self._gozer?)

        w = event.widget

        if self.placement == 'center':
            x = w.winfo_rootx()
            y = w.winfo_rooty() - self.menu.yposition( int(self._count / 2.))
        elif self.placement == 'n':
            x = w.winfo_rootx()
            y = w.winfo_rooty() - self.menu.yposition(self._count)            
        elif self.placement == 'ne':
            x = w.winfo_rootx() + w.winfo_width()/2
            y = w.winfo_rooty() - self.menu.yposition( int(self._count / 2.))
        elif self.placement == 's':
            x = w.winfo_rootx()
            y = w.winfo_rooty() - self.menu.yposition(self._count)            
        elif self.placement == 'close':
            x = 3
            y = -3
            self.menu.post(event.x_root+x, event.y_root+y)
        elif self.placement == 'right':
            #x_off = event.widget.1
            pass
            
        self._posted = True
        self.menu.after(50, lambda: self.menu.bind('<Leave>', self._gozer) )
        self.menu.post(x, y)

        """



class ProgressDialogWindow(RaccoonDefaultWidget , DebugTools.DebugObj):
    """ 
        parent          : Tk parent of the widget

        function        : function to be threaded

        func_args       : function arguments 

        func_kwargs     : function kw arguments 

        title           : title of the window

        message         : message to be shown in the window (LABEL)

        operation       : string with the description of the operation threaded
                          (used for the notification message text)

        image           : an ImageTk object ( or a file?) FIXME

        autoclose       : automatically close the dialog when threaded function 
                          successfully terminated

        cb              : callback passed to the threaded function, i.e. to perform updates

        progresstype    : description of the progress, that can be False (disabled) or
                          'percent' (return float percent) or 'count' (return integer)

        showtime        : Bool, show elapsed time
    
        debug           : enable debug

        USAGE: 

            the only requirement is that the function that is called should support
            as a parameter 'stopcheck' a function that returns True when the threaded
            function should stop (see self.start() )

                func_kwargs = {'path': "/", 'pattern': '*', 'recursive' : True}
                func = hf.pathToList
                progressWin = ProgressDialogWindow(parent, title, func, func_kwargs)
                progressWin.start()

    """
    def __init__(self, parent, function, func_args=(), func_kwargs={}, 
            title='ProgressDialogWindow', message='threaded operation', 
            operation = 'GenericProcessing', image = None, autoclose=False,
            cb = None, progresstype = None, showtime = True, debug=False):
        RaccoonDefaultWidget.__init__(self, parent)
        DebugTools.DebugObj.__init__(self, debug)
        self.title = title
        self._function = function
        self._args = func_args
        self._kwargs = func_kwargs
        self.image = image 
        self.message = message
        self.operation = operation
        self.autoclose = autoclose
        self.progresstype = progresstype
        self.showtime = showtime
        self.cb = cb
        self.queue = Queue.Queue()
        self.progress = 0
        self.pending = None
        self.status = 'not ready'
        self._STOP = False
        self._COMPLETED = False
        self._ERROR = None
        self._SLEEP = 0.1 # 075 # update interval (sec.)
        self.rolling = """-\|/""" # .oO@*  """
        #self.rolling = ".oO@*   "


    def stop(self, event=None):
        """ halt the thread"""
        t = 'Confirm'
        i = 'warning'
        m = 'Stop %s?' % self.operation
        if not tmb.askyesno(parent=self.win.interior(), title=t, icon=i, message=m):
            return
        self._STOP = True
        self._COMPLETED = False
        self.closingProcedure()
    
    def changeButton(self, event=None):
        """ update the text of the button when the thread ends/get stopped"""
        self.win.component('buttonbox').button(0).configure(text='Close', command=self.close)

    def getOutput(self):
        """return the output of the threaded function """
        if self.queue.empty():
            return []
        if self._ERROR:
            return []
        output = self.queue.get()
        return output

    def checkStop(self):
        """ function used to check if the thread must stop"""
        return self._STOP

    def close(self, event=None):
        """close the window"""
        self.win.destroy()


    def buildGUI(self):
        """ set the widgets shown"""
        self.win = Pmw.Dialog(self.parent, title=self.title, buttons=('Stop',), command=self.stop) 
        self.win.component('buttonbox').button(0).configure(font=self.FONT, default='disabled', **self.BORDER)
        #            TITLE
        # MESSAGE
        # Status : [ Running, stopped, error
        # Error:  xxxxxxxxxxxxx
        # Elapsed: 
        # Progress/process
        i = tk.Frame(self.win.interior(), relief = 'flat') #, bd=2) #, bg='white')
        # message
        f = tk.Frame(i) # , bg='white')
        if self.image:
            tk.Label(f, image = self.image).pack(anchor='w', side='left', expand=0, padx=6, pady=6)
        tk.Label(f, text = self.message, font=self.FONT).pack(anchor='w',side='left', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x',pady=5,padx=3)
        # status
        i.pack(expand=0,fill='both')

        i = tk.Frame(self.win.interior(), relief = 'sunken', bd=2, bg='white')
        # SPACER
        #tk.Frame(i, height=2, bd=2, relief='sunken',bg='black').pack(anchor='n', side='top', expand=0,fill='x')
        f = tk.Frame(i, bg='white')

        tk.Label(f, text = 'Status : ', width = 20, anchor='e',bg='white',
            font=self.FONTbold).pack(anchor='w',side='left', expand=0, fill='x')
        self.status_label = tk.Label(f, width = 30, text = 'ready', anchor='w', bg='white', font=self.FONT)
        self.status_label.pack(anchor='w',side='left', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x', padx=3)
        # elapsed time
        if self.showtime:
            f = tk.Frame(i, bg='white')
            tk.Label(f, text = 'Elapsed time : ', width = 20, anchor='e', bg='white',
                font=self.FONTbold).pack(anchor='w',side='left', expand=0, fill='x')
            self.time_label = tk.Label(f, width = 30, text = '00 : 00 : 00', anchor='w', 
                bg='white', font=self.FONT)
            self.time_label.pack(anchor='w',side='left', expand=0, fill='x')
            f.pack(side='top', anchor='w', expand=0, fill='x', padx=3)
        # error
        f = tk.Frame(i)
        self.error_title = tk.Label(f, text = ' ', fg='red', bg='white', width = 20, anchor='e', font=self.FONTbold)
        self.error_title.pack(anchor='w',side='left', expand=0, fill='x')
        self.error_label = tk.Label(f, width = 30, text = ' ', font=self.FONT, anchor='w', bg='white')
        self.error_label.pack(anchor='w',side='top', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x',padx=3)

        if self.progresstype == 'percent':
            #create percent bar
            f = tk.Frame(i, bg='white')
            self.progressBar = hf.ProgressBarThreadsafe(i, w = 300, h = 20)
            self.progressBar.pack(anchor='n', side='top', expand=0, fill='none')
            f.pack(side='top', anchor='n', expand=0, fill='none', padx=5, pady=8)
        elif self.progresstype == 'count':
            # create counter label
            # FIXME
            pass
        elif self.progresstype == None:
            # create the default "is alive" feedback
            self.dotsLabel = tk.Label(i, bg='white', text = self.rolling[0])
            self.dotsLabel.pack()
            pass
        i.pack(expand=0,fill='both')

    def setPercent(self, value):
        """update the percentage widget"""
        self.progress = value


    def _wrapped_function(self):
        """ the wrapped version of the function to be threaded
            with the FIFO queue used to export results
        """
        try:
        #if True:
            self._kwargs['stopcheck'] = self.checkStop
            if self.progresstype == 'percent':
                self._kwargs['showpercent'] = self.setPercent
            elif self.progresstype == 'count':
                #self._kwargs['showcount'] = self.setPercent
                pass
            res = self._function( *self._args, **self._kwargs)
            self.queue.put(res)
        except:
        #else:
            self._ERROR = sys.exc_info()[1]

    def start(self): 
        """ show the widget and start the threaded function 
        """
        self.buildGUI()
        # trigger the actual start...
        self.win.after(200, self._run)
        self.win.activate()
        
    def _run(self): 
        """ start the calculation and perform the GUI update"""
        self.status_label.configure(text = 'running')
        # create the thread and start it
        self.pending = threading.Thread(target = self._wrapped_function)
        self.pending.start()
        self.updateStatus()
        self._start_time = time.time()
        while self.status == 'running':
            time.sleep(self._SLEEP)
            self.updateStatus()
            self.updateGUI()
            self.updateGUI()

    def updateStatus(self):
        """ return the status of the job"""
        if self.pending == None:
            self.status = 'not started'
        elif self.pending.isAlive():
            self.status = 'running'
        elif not self._STOP:
            self._COMPLETED = True
            self.status = 'completed'
            self.closingProcedure()
        elif self._STOP:
            self.COMPLETED = False
            self.status = 'stopped'
            self.closingProcedure()

    def updateGUI(self):
        """ update the progress status"""
        try:
            self.parent.update()
            if self.showtime:
                # update time
                s = int(time.time() - self._start_time)
                h = s/3600
                remainder = s % 3600
                m = remainder/60
                s = remainder % 60
                timestr = "%02d : %02d : %02d" % (h,m,s)
                self.time_label.configure(text = timestr)
            if self.progresstype == 'percent':
                self.progressBar.set(self.progress)
                return
            elif self.progresstype == 'count':
                # update counter # FIXME
                return
            # provide the generic progress
            self.rolling = self.rolling[1:] + self.rolling[0]
            self.dotsLabel.configure(text = self.rolling[0])
        except tk.TclError:
            # possibly a latest update that was late
            pass
        except:
            print "[ unexpected error in RaccoonBasics, not dangerous... possibly...]"
            pass


    def closingProcedure(self):
        """ perform gui closing procedure when
            the threaded operation is completed
        """

        if not self._ERROR == None:
            # enable the error field
            self.error_title.configure(text='Error : ')
            self.error_label.configure(text=self._ERROR)
            status = 'error'
        
        elif self._COMPLETED:
            if self.autoclose and not self._ERROR:
                self.close()
                return
            status = 'done'
        elif self._STOP:
            status = 'stopped'

        self.status_label.configure(text = status)
        if self.progresstype == None:
            self.dotsLabel.configure(text='  ')
        self.changeButton()



       

class ProgressDialogWindowTk(RaccoonDefaultWidget , DebugTools.DebugObj):
    """ 
        parent          : Tk parent of the widget

        function        : function to be threaded

        func_args       : function arguments 

        func_kwargs     : function kw arguments 

        title           : title of the window

        message         : message to be shown in the window (LABEL)

        operation       : string with the description of the operation threaded
                          (used for the notification message text)

        image           : an ImageTk object ( or a file?) FIXME

        autoclose       : automatically close the dialog when threaded function 
                          successfully terminated

        cb              : callback passed to the threaded function, i.e. to perform updates

        progresstype    : description of the progress, that can be False (disabled) or
                          'percent' (return float percent) or 'count' (return integer)

        showtime        : Bool, show elapsed time
    
        debug           : enable debug

        USAGE: 

            the only requirement is that the function that is called should support
            as a parameter 'stopcheck' a function that returns True when the threaded
            function should stop (see self.start() )

                func_kwargs = {'path': "/", 'pattern': '*', 'recursive' : True}
                func = hf.pathToList
                progressWin = ProgressDialogWindow(parent, title, func, func_kwargs)
                progressWin.start()

    """
    def __init__(self, parent, function, func_args=(), func_kwargs={}, 
            title='ProgressDialogWindow', message='threaded operation', 
            operation = 'GenericProcessing', image = None, autoclose=False,
            cb = None, progresstype = None, showtime = True, debug=False):
        RaccoonDefaultWidget.__init__(self, parent)
        DebugTools.DebugObj.__init__(self, debug)
        self.title = title
        self._function = function
        self._args = func_args
        self._kwargs = func_kwargs
        self.image = image 
        self.message = message
        self.operation = operation
        self.autoclose = autoclose
        self.progresstype = progresstype
        self.showtime = showtime
        self.cb = cb
        self.status = 'not ready'
        self.result = None
        self._STOP = False
        self._COMPLETED = False
        self._ERROR = None
        self._SLEEP = 0.1 # 075 # update interval (sec.)
        self.rolling = """-\|/""" # .oO@*  """
        self.rolling_counter = 0
        self.rolling = unicode(u"\u2588\u2589\u258A\u258B\u258C\u258D\u258E\u258F") # horizontal
        #self.rolling = unicode(u"\u2581\u2582\u2583\u2584\u2585\u2586\u2587") # vertical
        #self.rolling = ".oO@*   "


    def stop(self, event=None):
        """ halt the thread"""
        t = 'Confirm'
        i = 'warning'
        m = 'Stop %s?' % self.operation
        if not tmb.askyesno(parent=self.win.interior(), title=t, icon=i, message=m):
            return
        self._STOP = True
        self._COMPLETED = False
        self.closingProcedure()
    
    def changeButton(self, event=None):
        """ update the text of the button when the thread ends/get stopped"""
        self.win.component('buttonbox').button(0).configure(text='Close', command=self.close)

    def getOutput(self):
        """return the output of the threaded function """
        #if self.queue.empty():
        #    return []
        #if self._ERROR:
        #    return []
        #output = self.queue.get()
        return self.result

    def checkStop(self):
        """ function used to check if the thread must stop"""
        return self._STOP

    def close(self, event=None):
        """close the window"""
        self.win.destroy()


    def buildGUI(self):
        """ set the widgets shown"""
        self.win = Pmw.Dialog(self.parent, title=self.title, buttons=('Stop',), command=self.stop) 
        button = self.win.component('buttonbox').button(0)
        button.configure(font=self.FONT, default='disabled', cursor='arrow', **self.BORDER)
        #            TITLE
        # MESSAGE
        # Status : [ Running, stopped, error
        # Error:  xxxxxxxxxxxxx
        # Elapsed: 
        # Progress/process
        i = tk.Frame(self.win.interior(), relief = 'flat') #, bd=2) #, bg='white')
        # message
        f = tk.Frame(i) # , bg='white')
        if self.image:
            tk.Label(f, image = self.image).pack(anchor='w', side='left', expand=0, padx=6, pady=6)
        tk.Label(f, text = self.message, font=self.FONT).pack(anchor='w',side='left', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x',pady=5,padx=3)
        # status
        i.pack(expand=0,fill='both')

        i = tk.Frame(self.win.interior(), relief = 'sunken', bd=2, bg='white')
        # SPACER
        f = tk.Frame(i, bg='white')
        tk.Label(f, text = 'Status : ', width = 20, anchor='e',bg='white',
            font=self.FONTbold).pack(anchor='w',side='left', expand=0, fill='x')
        self.status_label = tk.Label(f, width = 30, text = 'ready', anchor='w', bg='white', font=self.FONT)
        self.status_label.pack(anchor='w',side='left', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x', padx=3)
        # elapsed time
        if self.showtime:
            f = tk.Frame(i, bg='white')
            tk.Label(f, text = 'Elapsed time : ', width = 20, anchor='e', bg='white',
                font=self.FONTbold).pack(anchor='w',side='left', expand=0, fill='x')
            self.time_label = tk.Label(f, width = 30, text = '00 : 00 : 00', anchor='w', 
                bg='white', font=self.FONT)
            self.time_label.pack(anchor='w',side='left', expand=0, fill='x')
            f.pack(side='top', anchor='w', expand=0, fill='x', padx=3)
        # error
        f = tk.Frame(i)
        self.error_title = tk.Label(f, text = ' ', fg='red', bg='white', width = 20, anchor='e', font=self.FONTbold)
        self.error_title.pack(anchor='w',side='left', expand=0, fill='x')
        self.error_label = tk.Label(f, width = 30, text = ' ', font=self.FONT, anchor='w', bg='white')
        self.error_label.pack(anchor='w',side='top', expand=0, fill='x')
        f.pack(side='top', anchor='w', expand=0, fill='x',padx=3)

        if self.progresstype == 'percent':
            #create percent bar
            f = tk.Frame(i, bg='white')
            self.progressBar = hf.ProgressBarThreadsafe(i, w = 300, h = 20)
            self.progressBar.pack(anchor='n', side='top', expand=0, fill='none')
            f.pack(side='top', anchor='n', expand=0, fill='none', padx=5, pady=8)
        elif self.progresstype == 'count':
            # create counter label
            # FIXME
            pass
        elif self.progresstype == None:
            # create the default "is alive" feedback
            self.dotsLabel = tk.Label(i, bg='white', text = self.rolling[0])
            self.dotsLabel.pack()
            pass
        i.pack(expand=0,fill='both')

    def setPercent(self, value):
        """update the percentage widget"""
        self.progress = value

    def start(self):
        """ initiate the processing """
        # build the GUI
        self.buildGUI()
        # schedule the start function
        self.parent.after(100, self.run)
        # show gui
        self.win.activate()

    def getResults(self):
        """ set the complete variable to True"""
        return self.result

    def run(self,event=None):
        """ run the blocking function"""
        wrap_kwargs = { 'stopcheck' : self.checkStop,
                        'GUI'       : self.win.interior(),
                      }
        if self.progresstype == 'percent':
            wrap_kwargs['showpercent'] = self.progressBar.set
        self._kwargs.update(wrap_kwargs)
        self._start_time = time.time()
        # schedule an update
        self.win.interior().after(100, self.updateGUI )
        try:
            self.result = self._function(**self._kwargs)
            self._COMPLETED = True
        except:
            self._ERROR = sys.exc_info()[1]
            self._STOP = True
            print "CATCHED ERROR!", self._ERROR


    #XXX REDEFINE THE CLOSE PROTOCOL TO BE CLOSE()!!!!

    def updateGUI(self):
        """ update the progress status"""
        if self._COMPLETED or self._STOP:
            self.closingProcedure()
            return
        self.win.interior().after(100, self.updateGUI )
        if self.showtime:
            # update time
            s = int(time.time() - self._start_time)
            h = s/3600
            remainder = s % 3600
            m = remainder/60
            s = remainder % 60
            timestr = "%02d : %02d : %02d" % (h,m,s)
            self.time_label.configure(text = timestr)
        if self.progresstype:
            return
        #if self.progresstype == 'percent':
        #    self.progressBar.set(self.progress)
        #    return
        #elif self.progresstype == 'count':
        #    # update counter # FIXME
        #    return
        # provide the generic progress
        #if self.rolling_counter > len(self.rolling):
        #    self.rolling=self.rolling[::-1]
        #    self.rolling_counter = 0
        #    print "inverting"
        self.rolling = self.rolling[1:] + self.rolling[0]
        #string = " ".join(list(self.rolling[::-1]) )
        string = " ".join(list(self.rolling))
        self.dotsLabel.configure(text = string, fg='blue', bg='white') #, **self.BORDER) # [0])
        self.rolling_counter += 1


    def closingProcedure(self):
        """ perform gui closing procedure when
            the threaded operation is completed
        """

        testers = []
        if not self._ERROR == None:
            # enable the error field
            self.error_title.configure(text='Error : ')
            self.error_label.configure(text=self._ERROR)
            status = 'error'
        
        elif self._COMPLETED:
            if self.autoclose and not self._ERROR:
                self.close()
                return
            status = 'done'
        elif self._STOP:
            status = 'stopped'
        self.status_label.configure(text = status)
        if self.progresstype == None:
            self.dotsLabel.configure(text='  ')
        self.changeButton()


class About(RaccoonDefaultWidget):
    """ refined about window"""
    def __init__(self, parent=None, iconpath=None):
        RaccoonDefaultWidget.__init__(parent, iconpath)
        self.tabs = {}
        self.win = Pmw.Dialog(self.parent, title='About Raccoon2', buttons=('Close',))
        self.notebook = Pmw.NoteBook(self.parent)
        for tab in ['About', 'Funding & Acknowledgements']:
            self.tabs[tab] = self.notebook.add(tab)
        self.notebook.pack(expand=1, fill='both')
        self.notebook.setnaturalsize(self.tabs.keys())
        self.initImages()
        self.populateAbout()
        self.populateAuthors()
        self.populateFundingAck()
        self.win.activate()

    def initImages(self):
        """ initialize the icons for the interface"""
        images_path = CADD.Raccoon2.ICONPATH
        f = icon_path + os.sep + '.png'
        self._ICON_sys = ImageTk.PhotoImage(Image.open(f))



    
    def populateAbout(self):
        """ about raccoon"""
        text = ('AutoDock | Raccoon 2\n'
                'v1.0beta rev 0.1\n')
        pass

    def populateAuthors(self):
        """ set authors names"""
        authors = ['Stefano Forli, TSRI', 
                   'Arthur J. Olson, TSRI', 
                   'Michel Sanner, TSRI']

    def populateFundingAck(self):
        thanks = ['Lisa Dong', 'Jc Ducomm',
                  'Alex L. Perryman',
                  'Luca',
                  'Peter']
        pass






#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli, Michel Sanner
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import Tkinter, tkFont

from mglutil.util.callback import CallbackFunction

class RangeSlider(Tkinter.Frame):
    """
    - added method for setting the actual range values that can be returned by the slider
    - added getvalues method for getting current slider values
    - added min/max labels to the range
    - minor esthetic changes

    # TODO 
    - the widget is not tolearant with other non-pack placement managers? (i.e. grid?)
            remove the Tkinter packing configuration?

    """
    def __init__(self, master=None, width=100, height=15, vmin=0, vmax=100, cb=None,
                 canvasCfg={}):

        self.cb = cb
        self.vmin = vmin
        self.vmax = vmax
        self.currmin = self.vmin
        self.currmax = self.vmax

        family = 'Arial'
        #family = 'Helvetica'
        self.FONT = tkFont.Font(family=family,size=9)
        self.FONTbold = tkFont.Font(family=family,size=9,weight="bold")
        self.FONTsmall = tkFont.Font(family=family,size=height/2)
        self.BORDER = { 'bd':1,'highlightbackground':'black',
                    'borderwidth':2,'highlightcolor':'black','highlightthickness':1}

        self.minval = Tkinter.Label(master, text=self.vmin, anchor='e', width=8, padx=0, pady=0,
            font=self.FONTbold)
        self.minval.pack(side='left', anchor='w',padx=2)

        sett = self.BORDER.copy()
        sett['highlightbackground'] = 'gray40'

        Tkinter.Frame.__init__(self, master, **sett)
        Tkinter.Pack.config(self, side='left', anchor='w')

        self.maxval = Tkinter.Label(master, text=self.vmax, anchor='w', width=8, padx=0, pady=0,
            font=self.FONTbold)
        self.maxval.pack(side='left', anchor='w',padx=2)

        self.width = width
        self.height = height
        self.canvasCfg = canvasCfg
        self.orient = 'horizontal'
        self.createCanvas(master)


    def setrange(self, vmin=None, vmax=None):
        """ defines the value of the min and max allowed values"""
        #print "SLIDERSETRANGE>", vmin, vmax
        if not vmin == None:
            self.vmin = self.currmin = vmin
        if not vmax == None:
            self.vmax = self.currmax = vmax
        self.minval.configure(text="%2.3f" % self.vmin)
        self.maxval.configure(text="%2.3f" % self.vmax)
        self.reset()

    def reset(self, event=None):
        """ reset the sliders to the far edges"""
        canvas = self.canvas
        bb = canvas.bbox('left')
        if self.orient=='horizontal':
            canvas.move('left', self.leftOrigBB[0]-bb[0], 0)
        else:
            canvas.move('left', 0, bb[1]-self.leftOrigBB[1])

        bb = canvas.bbox('right')
        if self.orient=='horizontal':
            canvas.move('right', self.rightOrigBB[2]-bb[2], 0)
        else:
            canvas.move('right', 0, bb[3]-self.rightOrigBB[3])
        self.updateBackPoly(event)

##        c = self.canvas
##        bb1 = c.bbox('left')
##        bb2 = c.bbox('right')
##        print "BB1", bb1
##        print "BB2", bb2
##        print "\n\n\n\n ASK MICHEL!!!"
##        #self.canvas.move('left', 0, 0)
##        #self.canvas.move('right', 0, self.width) #-40, 0)
##

    def mouseDown(self, tag, event=None):
        if self.orient=='horizontal':
            self.lastx = event.x
        else:
            self.lastx = event.y


    def restrictHorizontalMotion(self, tag, dx):
        # tag wants to move by dx
        canvas = self.canvas
        bb1 = canvas.bbox('left')
        bb2 = canvas.bbox('right')        
        if tag=='left':
            if bb1[0]+dx <= 0:
                return -bb1[0]
            if bb1[2]+dx >= bb2[0]:
                return dx - (bb1[2]+dx-bb2[0])
            return dx
        if tag=='right':
            if bb2[2]+dx >= self.width+self.bw:
                return self.width - bb2[2] + self.bw
            if bb2[0]+dx <= bb1[2]:
                return dx - (bb2[0]+dx-bb1[2])
            return dx

        if tag=='left||right':
            if bb1[0]+dx <= self.bw:
                return -bb1[0]
            if bb2[2]+dx >= self.width + self.bw:
                return self.width-bb2[2] + self.bw
            return dx

        
    def restrictVerticalMotion(self, tag, dx):
        print 'NOT IMPLEMENTED'


    def mouseMove(self, tag, event=None):
        if tag=='poly':
            tag = "left||right"
        canvas = self.canvas
        if self.orient=='horizontal':
            dx = event.x - self.lastx
            self.lastx = event.x
            # make sure we keep in bounds
            dx1 = self.restrictHorizontalMotion(tag, dx)
            self.canvas.move(tag, dx1, 0)
            #print 'AAA', canvas.bbox(tag)
        else:
            dx = event.y - self.lastx
            self.lasty = event.y
            dx1 = self.restrictVerticalMotion(tag, dx)
            self.canvas.move(tag, 0, dx1)
        self.updateBackPoly()
        

    def mouseUp(self, tag, event=None):
        pass
    

    def createCanvas(self, master):
        height = self.height
        width = self.width

        cd={'width':width, 'height':height, 'relief':'sunken', 'borderwidth':0,
            'bg':'white'}
            #'bg':'grey70'}
        cd.update(self.canvasCfg)
        canvas = self.canvas = Tkinter.Canvas(self, **cd)
        self.canvas.configure(cursor='sb_h_double_arrow')

        self.bw = bw = cd['borderwidth']

        # create the background  polygon between the cursors
        # bb of poly is (0, 0, 200, 16)
        lp1 = canvas.create_rectangle( 1+bw, 1+bw, width+bw-1, height+bw,
                                       fill='SteelBlue1', outline='black',
                                       #fill='#0081FF', outline='black',
                                       tags=('poly',))

        # create left cursor (light upper left line)
        ## bb for 'leftl is (0,0,20,16)
        lc1 = canvas.create_line( 2+bw, height+bw-1, 2+bw, 2+bw, 18+bw, 2+bw,
                                  fill='white', width=1, tags=('left',))

        # create a polygon filling the cursor
        lp1 = canvas.create_rectangle( bw+3, bw+3, 17+bw, height+bw-2,
                                       fill='grey75', outline='grey75',
                                       tags=('left',), width=1)

        # create left cursor (dark lower rigth line)
        lc2 = canvas.create_line( 18+bw, 2+bw, 18+bw, height+bw-1, 2+bw, 
                                  height+bw-1, width=1, fill='black', tags=('left',))

        # create a letter in the cursor
        ll1 = canvas.create_text( bw+11, 1+bw+self.height/2, text=unicode(u"\u25B6"), font=self.FONTsmall, tags=('left',))
        self.leftOrigBB = canvas.bbox('left')
        
        # create right cursor (light upper left line)
        rc1 = canvas.create_line( 2+bw, height+bw-1, 2+bw, 2+bw, 18+bw, 2+bw,
                                  fill='white', tags=('right',))
        # create a polygon filling the cursor
        rp1 = canvas.create_rectangle( bw+3, bw+3, 17+bw, height+bw-2,
                                       fill='grey75', outline='grey75',
                                       tags=('right',))
        # create right cursor (dark lower rigth line)
        rc2 = canvas.create_line( 18+bw, 2+bw, 18+bw, height+bw-1, 2+bw,
                                  height+bw-1, fill='black', tags=('right',))
        # create a letter in the cursor
        rl1 = canvas.create_text( bw+11, bw+self.height/2, text=unicode(u"\u25C0"), font=self.FONTsmall, tags=('right',))
        self.canvas.move( 'right', width-20, 0)
        self.rightOrigBB = canvas.bbox('right')

        self.updateBackPoly()

        canvas.tag_bind('left', "<ButtonPress-1>",
                        CallbackFunction(self.mouseDown, 'left'))
        #canvas.tag_bind('left', "<ButtonRelease-1>",
        #                CallbackFunction( self.mouseUp, 'left'))
        canvas.tag_bind('left', "<B1-Motion>",
                        CallbackFunction( self.mouseMove, 'left'))

        canvas.tag_bind('right', "<ButtonPress-1>",
                        CallbackFunction( self.mouseDown, 'right'))
        #canvas.tag_bind('right', "<ButtonRelease-1>",
        #                CallbackFunction( self.mouseUp, 'right'))
        canvas.tag_bind('right', "<B1-Motion>",
                        CallbackFunction( self.mouseMove, 'right'))

        canvas.tag_bind('poly', "<ButtonPress-1>",
                        CallbackFunction( self.mouseDown, 'poly'))
        canvas.tag_bind('poly', "<B1-Motion>",
                        CallbackFunction( self.mouseMove, 'poly'))
        #canvas.tag_bind('poly', "<ButtonRelease-1>",
        #                CallbackFunction( self.mouseUp, 'poly'))

        self.canvas.pack(side=Tkinter.LEFT)


    def updateBackPoly(self, event=None):
        """update the background polygon to span betweent the cursors"""
        canvas = self.canvas
        bb1 = canvas.bbox('left')
        bb2 = canvas.bbox('right')
        canvas.coords('poly', bb1[2]+1, bb1[1]+1, bb2[0]-1, bb1[3]-1)
        self.currmin = self.vmin + (self.vmax - self.vmin) * self.pc(bb1[2])
        self.currmax = self.vmin + (self.vmax - self.vmin) * self.pc(bb2[0])
        if self.cb:
            try:
                self.cb(self.currmin, self.currmax)
            except:
                # this is here to catch when the widget 
                #is packed the first time
                #print "catched exception with callback... first-time init?"
                return
        
    def pc(self, value):
        if value == 0:
            return 0
        return (float(value-self.bw-20)/float(self.width-40)) #*100
        
    def getvalues(self):
        """ return the current slider values"""
        return (self.currmin, self.currmax)
        

if __name__=='__main__':
    root = Tkinter.Tk()
    rangeSlider = RangeSlider(root, width=200, height=15)
    #rangeSlider2 = RangeSlider(root, width=200, height=15)
    print rangeSlider.canvas.bbox('left')
    print rangeSlider.canvas.bbox('right')
    print rangeSlider.canvas.bbox('poly')



#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import CADD.Raccoon2
import CADD.Raccoon2.HelperFunctionsN3P as hf
import RaccoonBasics as rb
import RaccoonEvents
import DebugTools
import os, Pmw, glob, sys
from PmwOptionMenu import OptionMenu as OptionMenuFix
import Tkinter as tk
import tkMessageBox as tmb
import tkFileDialog as tfd
from PIL import Image, ImageTk

# mgl modules
from mglutil.events import Event, EventHandler
from mglutil.util.callback import CallbackFunction #as cb


# TODO add an arbitrary distance constraint?
# TODO add pharmacophore score


class InteractionFilterWidget(rb.RaccoonDefaultWidget, DebugTools.DebugObj, tk.Frame):
    """ provide the interaction filter widget
        with different types:

            HBa
            HBd
            HB
            Metal
            vdW
            Pi-stack
            T-stack
            Generic stack
           
   checkbutton   pulldown     entry    button (remove)
        [x] | | [type..] | [ string ] [ x ]
    """
    def __init__(self, parent, manager, settings={}, color=None, debug = False): #, destroy_cb):
        self.color = '#bbff33'
        self.color = color # differenciate filter types?
        rb.RaccoonDefaultWidget.__init__(self, parent) 
        DebugTools.DebugObj.__init__(self, debug)
        tk.Frame.__init__(self, master=self.parent, relief='raised', bg=color,  **self.BORDER)
        self.manager = manager
        self._widgets = [self]

        self.initIcons()
        self.destroy_cb = None # destroy_cb
        self.eventManager = self.manager.eventManager
        #self.eventManager = eventManager
        #self.event = RaccoonEvents.FilterSetSelection()

        self.settings = settings
        self.build()
        self.setfilter()

        # hack to have mouse wheel to work all the time
        for w in self._widgets:
            w.bind("<Button-4>", self.manager.mousescroll)
            w.bind("<Button-5>", self.manager.mousescroll)
            if hasattr(w, 'components'):
                for c in w.components():
                    w.component(c).bind("<Button-4>", self.manager.mousescroll)
                    w.component(c).bind("<Button-5>", self.manager.mousescroll)


    def build(self):
        """ build the widgets"""
        entry_width = 10

        # types strings
        self._notype = '< select type >'
        self.stringToType = { 
                        'HB donor' : 'hbd',
                        'HB acceptor' : 'hba',
                        'HB any' : 'hb',
                        'Metal coord': 'metal',
                        'vdW contact': 'vdw',
                        'Pi-stacking': 'ppi', # XXX check this
                        'T-stacking' : 'tpi', # XXX check this
                        'Any stacking': 'pi', # XXX check this
                    }
        self.typeToString = {}
        for k,v in self.stringToType.items():
            self.typeToString[v] = k

        # active checkbutton
        self.active_var = tk.BooleanVar(value=False)
        self.active = tk.Checkbutton(self, variable=self.active_var, 
            bg=self.color, command=self.trigger)
        #self.active.pack(side='left', anchor='w', expand=0, fill='none')

        # interaction type
        self.typeChoice = OptionMenuFix(self,
            menubutton_font = self.FONT,
            menubutton_bg = self.color,
            menu_font = self.FONT,
            menubutton_width = 15,
            menubutton_bd = 1, menubutton_highlightbackground = 'black',
            menubutton_borderwidth=1, menubutton_highlightcolor='black', 
            menubutton_highlightthickness = 1,
            menubutton_height=1,
            command = self.trigger, # self.typeValidator, # self.trigger,
            items = sorted(self.stringToType.keys())
            )
        self.typeChoice.setvalue(self._notype)
        self.typeChoice.pack(side='left', anchor='w', expand=1, fill='x', padx=3, pady=1)
        self._widgets.append(self.typeChoice)

        # strings
        f = tk.Frame(self) # nested frames to have a decent alignment (Tk sucks(TM) )
        ef = tk.Frame(f) 
        # chain
        self.chain = Pmw.EntryField(ef, entry_font=self.FONT,
            labelpos = 'n', label_font=self.FONT, label_text='chain',
            hull_bg = self.color,
            entry_width=4, entry_justify='center',
            entry_bd = 1, entry_highlightbackground = 'black',
            entry_borderwidth=1, entry_highlightcolor='black', 
            entry_highlightthickness = 1,
            )
        self.chain.pack(side='left', anchor='n', expand=0, fill='x', padx=3 )
        self.chain.component('entry').bind('<Return>', self.return_cb)
        self.chain.component('entry').bind('<Leave>', self.focusLeave_cb)
        self._widgets.append(self.chain)

        # residue
        self.residue = Pmw.EntryField(ef, entry_font=self.FONT,
            labelpos = 'n', label_font=self.FONT, label_text='residue',
            hull_bg = self.color,
            entry_width=entry_width, entry_justify='center',
            entry_highlightbackground = 'black',
            entry_borderwidth=1, entry_highlightcolor='black', 
            entry_highlightthickness = 1,
            )
        self.residue.pack(side='left', anchor='n', expand=1, fill='x', padx=3)
        self.residue.component('entry').bind('<Return>', self.return_cb)
        self.residue.component('entry').bind('<Leave>', self.focusLeave_cb)
        self._widgets.append(self.residue)

        # atom
        self.atom = Pmw.EntryField(ef, entry_font=self.FONT,
            labelpos = 'n', label_font=self.FONT, label_text='atom',
            hull_bg = self.color,
            entry_width=entry_width, entry_justify='center',
            entry_highlightbackground = 'black',
            entry_borderwidth=1, entry_highlightcolor='black', 
            entry_highlightthickness = 1,
            )
        self.atom.pack(side='left', anchor='n', expand=1, fill='x', padx=3)
        self.atom.component('entry').bind('<Return>', self.return_cb)
        self._widgets.append(self.atom)

        s = tk.Frame(f, height=15)
        s.pack(side='bottom', anchor='w', expand=1, fill='both')

        ef.pack(side='top', anchor='w', expand=1, fill='x', padx=3)
        f.pack(side='left', anchor='w', expand=1, fill='both')

        self._widgets.append(s)
        self._widgets.append(ef)
        self._widgets.append(f)


        # wanted/not wanted
        self.wanted = True
        self.wantedButton = tk.Button(self, font=self.FONTbold, image=self._ICON_filtoff, relief='flat')
        selmenu = [ None,
                    ['wanted', 'normal', self.setPositive],
                    ['not wanted', 'normal', self.setNegative],
                    ['disable', 'normal', self.setOff],
                  ]
        menu = rb.RacMenu(self.wantedButton, selmenu, placement='under')
        self.wantedButton.pack(side='left', anchor='w', expand=0, fill='none', padx=1, pady=1)
        self._widgets.append(self.wantedButton)


        # destroy button
        self.destruction = tk.Button(self, text="X", image=self._ICON_del, command=self.destroy,
            bg=self.color,
            relief='flat') #,width=22) #, **self.BORDER)
        self.destruction.pack(side='left', anchor='w', expand=0, fill='none')
        self._widgets.append(self.destruction)


    def initIcons(self):
        """ initialize the icons for the interface"""
        icon_path = CADD.Raccoon2.ICONPATH

        f = icon_path + os.sep + 'removeSmall.png'
        f = icon_path + os.sep + 'removeSmallbw.png'
        self._ICON_del = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'filt_positive.png'
        f = icon_path + os.sep + 'filt_positiveLight.png'
        self._ICON_filtpos = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'filt_negativeLight.png'
        self._ICON_filtneg = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'filt_inactiveLight.png'
        self._ICON_filtoff = ImageTk.PhotoImage(Image.open(f))


    def setPositive(self, event=None):
        """ set the interaction to be wanted"""
        self.wanted = True
        self.wantedButton.configure(image=self._ICON_filtpos, text='A')
        self.enable()
        #self.trigger()

    def setNegative(self, event=None):
        """ set the interaction to be not wanted"""
        self.wanted = False
        self.wantedButton.configure(image=self._ICON_filtneg, text='R')
        self.enable()
        #self.trigger()

    def setOff(self, event=None):
        """ turn off the filter"""
        self.wantedButton.configure(image=self._ICON_filtoff, text='R')
        current = self.active_var.get()
        self.active_var.set(False)
        # the filter was active, it has been disabled
        # so we need to send event for filterEngine to
        # be triggered
        if not self.active_var.get() == current:
            self.sendEvent()
        
    def return_cb(self, event=None):
        """ function called when return is pressed
            in the text fields
        """
        if self.wanted == True:
            self.setPositive()
        else:
            self.setNegative()

    def focusLeave_cb(self, event=None):
        """ trigger status check for entries
            when they loose focus
            Important when the filter has been alread
            activated, then the user changes the text in the
            field
        """
        if not self.isActive():
            return
        self.entryValidator(quiet=False)

    def isActive(self, event=None):
        """ return if the filter is active"""
        return self.active_var.get()

    def disable(self, event=None):  
        """ disable the filter"""
        self.active_var.set(False)

    def enable(self, event=None, quiet=True):  
        """ enable the filter"""
        self.active_var.set(True)
        self.trigger(quiet=quiet)

    def entryValidator(self, string=None, quiet=True):
        """ validate the text entry field
        """
        #if string == None:
        #    string = self.entry.getvalue()
        chain = self.chain.getvalue().strip()
        res = self.residue.getvalue().strip()
        atom = self.atom.getvalue().strip()
        if not chain and not res and not atom:
            self.setOff()
            if not quiet:
                t = 'Interaction specification not valid'
                i = 'error'
                m = ('The interaction must be defined by typing at '
                     'least a chain, residue or atom string.\n'
                     'Single character "?" and multi-character "*" wildcards can be used.'
                    ) 
                tmb.showinfo(parent=self, title=t, icon=i, message=m)
            return False
        return True


    def typeValidator(self, string=None, quiet=False):
        """ validate type selection """
        _type = self.typeChoice.getvalue()
        if _type == self._notype:
            if not quiet:
                t = 'Filter type error'
                i = 'warning'
                m = ('An interaction type must be selected')
                tmb.showinfo(parent=self, title=t, icon=i, message=m)
            self.setOff()
            return False        
        return True

    def isvalid(self, quiet=True):
        """return if the entry is valid"""
        typeCheck = self.typeValidator(quiet=quiet) 
        if not typeCheck: # or not self.isActive():
            return
        if not self.isActive():
            return
        entryCheck = self.entryValidator(quiet=quiet)
        return (typeCheck and entryCheck)

    def trigger(self, event=None, quiet=False):
        """ command executed when the filter is activated"""
        #curr_state = self.isActive() # active_var.get()
        if not self.isvalid(quiet=quiet):
            return
        self.active_var.set(True)
        self.sendEvent()

    def sendEvent(self):
        """ trigger the event"""
        e = RaccoonEvents.FilterInteractionEvent()
        self.eventManager.dispatchEvent(e)

    def getType(self):
        """ return interaction type"""
        return self.stringToType[ self.typeChoice.getvalue() ]

    def getPattern(self):
        """ return interaction text pattern"""
        c = self.chain.getvalue().strip()
        r = self.residue.getvalue().strip()
        a = self.atom.getvalue().strip()
        if c == "": c = "*"
        if r == "": r = "*"
        if a == "": a = "*"
        return "%s:%s:%s" % (c,r,a)


    def setfilter(self, settings={}):
        """ initialize filter with requested values"""
        if settings == {}:
            settings == self.settings
        if settings == {}:
            return
        t = self.settings['type']
        t = self.typeToString[t]
        e = self.settings['patter']
        self.typeChoice.setvalue(t)
        self.entry.setvalue(e)
        if self.isvalid(): self.active_var.set(True)
        # FIXME set this to active?
        #return { 'type' : t, 'pattern' : e }



    def destroy(self, event=None):
        """ remove itself from the list of filters
            and unpack the widget
        """
        t = 'Delete filter'
        i = 'info'
        m = ('Remove this filter?')
        if self.isvalid():
            if not tmb.askyesno(parent=self, title=t, message=m, icon=i):
                return
        self.destroy_cb()


class InteractFiltManager(rb.RaccoonDefaultWidget, DebugTools.DebugObj, Pmw.Group):
    """ provide the interaction filters manager widget

    """
    def __init__(self, parent, update_cb=None, debug=False): #, destroy_cb):

        rb.RaccoonDefaultWidget.__init__(self, parent)
        DebugTools.DebugObj.__init__(self, debug)
        Pmw.Group.__init__(self, parent=parent, tag_text = 'Target interactions', tag_font=self.FONTbold,
        
              ring_border=1, ring_highlightcolor='black', ring_highlightbackground='black',
            ring_highlightthickness=1,ring_relief='flat')      
        
        self.eventManager = RaccoonEvents.RaccoonEventManager()
        self.eventManager.registerListener(RaccoonEvents.FilterInteractionEvent, self.update)
        self.initIcons()
        self.filters = []
        self.update_cb = update_cb

        self.container = tk.Frame(self.interior()) # this contains the widgets on the left

        # build toolbar
        self.toolbar = tk.Frame(self.container)
        # add button
        b = tk.Button(self.toolbar, text='+', image=self._ICON_add, command=self.add, **self.BORDER)
        b.pack(anchor='w', side='left')
        # remove button
        b = tk.Button(self.toolbar, text='-', image=self._ICON_del, **self.BORDER)
        b.pack(anchor='w', side='left',padx=1)
        selmenu = [ None,
                    ['Remove inactive', 'normal', self.removeInactive],
                    ['Remove all', 'normal', self.nuke],
                  ]
        menu = rb.RacMenu(b, selmenu, placement='under')



        # selection button menu
        b = tk.Button(self.toolbar, text='s', image=self._ICON_selmenu, **self.BORDER)
        b.pack(anchor='w', side='left',padx=0)
        selmenu = [ None,
                    ['Activate all', 'normal', self.setAllActive],
                    ['Deactivate all', 'normal', self.setAllInactive],
                    ['Invert active/inactive', 'normal', self.invertStatus],
                  ]
        menu = rb.RacMenu(b, selmenu, placement='under')

        # mode pulldown
        self._mode_choices = { 'match all': 'all', 'match any': 'any'}
        self.modeChoice = OptionMenuFix(self.toolbar,
            menubutton_font = self.FONT,
            menu_font = self.FONT,
            menubutton_width = 10,
            menubutton_bd = 1, menubutton_highlightbackground = 'black',
            menubutton_borderwidth=1, menubutton_highlightcolor='black', 
            menubutton_highlightthickness = 1,
            menubutton_height=1,
            #command = self.trigger,
            items = sorted(self._mode_choices.keys(), reverse=True),
            )
        f = tk.Frame(self.toolbar)
        self.labelActiveFilt = tk.IntVar(value='0')
        tk.Label(f, text='active filters:', font=self.FONT).pack(anchor='w',side='left')
        tk.Label(f, textvar=self.labelActiveFilt, font=self.FONTbold).pack(anchor='w',side='left')
        f.pack(anchor='w', side='left', padx=15)
        self.modeChoice.pack(anchor='w', side='right', padx=1)
        self.toolbar.pack(anchor='w', side='top', expand=1, fill='x', padx=4, pady=2)

        # build filter filtFrame
        self.filtFrame = Pmw.ScrolledFrame(self.container, horizflex='expand', # vertflex='expand', 
                vscrollmode='static', hscrollmode='dynamic')
        #self.filtFrame.component('hull').configure(**self.BORDER)
        self.filtFrame.component('clipper').configure(bg='white')
        self.filtFrame.component('frame').configure(relief='flat', bg='white')
        #self.filtFrame.component('vertscrollbar').bind("<Button-4>", self.mousescroll)
        for c in [ 'borderframe', 'clipper', 'frame', 'horizscrollbar', 'hull', 'vertscrollbar']:
            self.filtFrame.component(c).bind("<Button-4>", self.mousescroll)
            self.filtFrame.component(c).bind("<Button-5>", self.mousescroll)
        self.interior().bind("<Button-4>", self.mousescroll)
        self.interior().bind("<Button-5>", self.mousescroll)

        self.filtFrame.pack(anchor='w', side='top', expand=1, fill='both',padx=4, pady=2)

        self.container.pack(anchor='w',side='left', expand=1, fill='both')

        self.percentageFrame = tk.Frame(self.interior()) # this contains the widgets on the right

        # pie
        self.pie = hf.PercentPie(self.percentageFrame, radius=50)
        self.pie.pack(anchor='n', side='top', padx=5, pady=5)
        #self.pie.set_percent(20)

        # passed label
        f = tk.Frame(self.percentageFrame)
        tk.Label(f, text='passed :', font=self.FONTbold).pack(side='left',anchor='w')
        self.statusLabel = tk.Label(f, text='0', width=8, anchor='w', font=self.FONT)
        self.statusLabel.pack(side='left', anchor='w')
        f.pack(anchor='n', side='top')

        #self.percentageFrame.pack(anchor='e',side='left', expand=1, fill='y')
        self.percentageFrame.pack(expand=0,fill=None, anchor='e', side='left')

    def setpassed(self, count, percentage):
        """ set the numerical value of 
            items that passed this filter
        """
        self.statusLabel.configure(text=count)
        self.pie.set_percent(percentage)    


    def setAllActive(self, event=None):
        """ set all filters active"""
        change = False
        for o in self.filters:
            if not o.isActive():
                o.enable(quiet=True)
                if o.isActive():
                    change = False
        if change:
            e = RaccoonEvents.FilterInteractionEvent()
            self.eventManager.dispatchEvent(e)

    def setAllInactive(self, event=None):
        """ set all filters inactive"""
        change = False
        for o in self.filters:
            if o.isActive():
                o.disable()
                change = True
        if change:
            e = RaccoonEvents.FilterInteractionEvent()
            self.eventManager.dispatchEvent(e)        

    def invertStatus(self, event=None):
        """ invert filter status"""
        change = False
        for o in self.filters:
            if o.isActive():
                o.disable()
                change = True
            else:
                o.enable(quiet=True)
                if o.isActive():
                    change = True
        if change:
            e = RaccoonEvents.FilterInteractionEvent()
            self.eventManager.dispatchEvent(e)


    def countActive(self, event=None):
        """ update count label of active filters"""
        c = 0
        for o in self.filters:
            if o.active_var.get():
                c+= 1
        self.labelActiveFilt.set(c)

    def mousescroll(self, event):
        """ manage mousescroll events"""
        if event.num == 5 or event.delta == -120:
            self.filtFrame.yview(tk.SCROLL, +1, tk.UNITS)
        if event.num ==4 or event.delta == 120:
            self.filtFrame.yview(tk.SCROLL, -1, tk.UNITS)



    def initIcons(self):
        """ initialize the icons for the interface"""
        icon_path = CADD.Raccoon2.ICONPATH

        f = icon_path + os.sep + 'removeSmall.png'
        self._ICON_del = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'addSmall.png'
        self._ICON_add = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'select_menu.png'
        self._ICON_selmenu = ImageTk.PhotoImage(Image.open(f))
        
        
    def add(self, event=None, settings={}):
        """ add a new filter widget
            if settings is not empty, the new filter
            will be initialized with the specified values
        """
        f = InteractionFilterWidget( self.filtFrame.interior(), 
            manager = self, settings=settings)
        cb = CallbackFunction(self.destroy, f)
        f.destroy_cb = cb
        self.filters.append(f)
        f.pack(anchor='w', side='top', padx=2, pady=1, expand=1,fill='x')
        # NOTE no event when adding (empty filter!) nor when 
        #      settings are restored, otherwise restoting a pre-saved
        #      set of multi-interaction settings will trigger 
        #      a lot of filter updates (i.e. troubles, when many ligands
        #      are loaded.
        #       the function creating 
        #      the filter (i.e. FiltSettingManager) should
        #      take care of populating all filters first
        #      *then* initial kick is given.
        # trigger event # XXX no! it's empty
        #if not settings == {}
        #    e = RaccoonEvents.FilterInteractionEvent()
        #    self.eventManager.dispatchEvent(e)

    def nuke(self, event=None, quiet=False):
        """ remove all the filters at once"""
        if len(self.filters) ==0 : return
        if not quiet:
            for o in self.filters:
                if o.isActive():
                    t = 'Remove all filters'
                    i = 'info'
                    m = ('All interaction filters are going to be removed.\n\nAre you sure?')
                    if not tmb.askyesno(parent=self.interior(), title=t, message=m, icon=i):
                        return
                    break
        for o in self.filters:
            o.pack_forget()
        self.filters = []
        e = RaccoonEvents.FilterInteractionEvent()
        self.eventManager.dispatchEvent(e)

    def removeInactive(self, event=None):
        """ remove inactive filters"""
        if len(self.filters) ==0 : return
        rem = []
        for o in self.filters:
            if not o.isActive():
                o.pack_forget()
                rem.append(o)
        for o in rem:
            self.filters.remove(o)



    def destroy(self, obj, event=None):
        """ remove a filter from the filters db
            and destroy the widget
        """
        obj.pack_forget()
        self.filters.remove(obj)
        e = RaccoonEvents.FilterInteractionEvent()
        self.eventManager.dispatchEvent(e)


    def getvalues(self, event=None):
        """ return the dictionary of all filters and the mode"""
        settings = { 'mode' : self._mode_choices[ self.modeChoice.getvalue() ] ,
                'pattern': {} }
        for o in self.filters:
            if o.isActive():
                otype = o.getType()
                opatt = o.getPattern()
                wanted = o.wanted
                if not otype in settings['pattern'].keys():
                    settings['pattern'][otype] = []
                self.dprint("iteractFilt[%s]  = [%s][%s]" % (otype, opatt, wanted))
                settings['pattern'][otype].append((opatt,wanted))
        return settings

    def update(self, event=None):
        """ when updating, fire this"""
        self.countActive()
        self.update_cb()



#!/bin/ksh ~/.mgltools/pythonsh
########################################################################
#
#    Vision Network - Python source code - file generated by vision
#    Wednesday 20 July 2011 18:37:15 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/CADD/workflows/MDanalysis/TrajQRClustering_0.1_net.py,v 1.2 2011/07/21 01:45:04 jren Exp $
#
# $Id: TrajQRClustering_0.1_net.py,v 1.2 2011/07/21 01:45:04 jren Exp $
#


if __name__=='__main__':
    from sys import argv
    if '--help' in argv or '-h' in argv or '-w' in argv: # run without Vision
        withoutVision = True
        from Vision.VPE import NoGuiExec
        ed = NoGuiExec()
        from NetworkEditor.net import Network
        import os
        masterNet = Network("process-"+str(os.getpid()))
        ed.addNetwork(masterNet)
    else: # run as a stand alone application while vision is hidden
        withoutVision = False
        from Vision import launchVisionToRunNetworkAsApplication, mainLoopVisionToRunNetworkAsApplication
	if '-noSplash' in argv:
	    splash = False
	else:
	    splash = True
        masterNet = launchVisionToRunNetworkAsApplication(splash=splash)
    import os
    masterNet.filename = os.path.abspath(__file__)
from traceback import print_exc
## loading libraries ##
from AutoDockTools.VisionInterface.Adt import Adt
from WebServices.VisionInterface.WSNodes import wslib
from Vision.StandardNodes import stdlib
try:
    masterNet
except (NameError, AttributeError): # we run the network outside Vision
    from NetworkEditor.net import Network
    masterNet = Network()

masterNet.getEditor().addLibraryInstance(Adt,"AutoDockTools.VisionInterface.Adt", "Adt")

masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")

masterNet.getEditor().addLibraryInstance(stdlib,"Vision.StandardNodes", "stdlib")

from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
try:
    addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
except:
    pass
try:
    ## saving node Directory_Containing_Input_PDB_Files ##
    from Vision.StandardNodes import DirBrowserNE
    Directory_Containing_Input_PDB_Files_0 = DirBrowserNE(constrkw={}, name='Directory_Containing_Input_PDB_Files', library=stdlib)
    masterNet.addNode(Directory_Containing_Input_PDB_Files_0,45,40)
    Directory_Containing_Input_PDB_Files_0.inputPortByName['directory'].widget.set(r"TrajQRClustering_0.1_input", run=False)
    apply(Directory_Containing_Input_PDB_Files_0.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named Directory_Containing_Input_PDB_Files in network masterNet"
    print_exc()
    Directory_Containing_Input_PDB_Files_0=None

try:
    ## saving node RMSD ##
    from Vision.StandardNodes import EntryNE
    RMSD_1 = EntryNE(constrkw={}, name='RMSD', library=stdlib)
    masterNet.addNode(RMSD_1,379,44)
    RMSD_1.inputPortByName['entry'].widget.set(r"0.5", run=False)
    apply(RMSD_1.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore EntryNE named RMSD in network masterNet"
    print_exc()
    RMSD_1=None

try:
    ## saving node Output_Directory_Path ##
    from Vision.StandardNodes import DirBrowserNE
    Output_Directory_Path_2 = DirBrowserNE(constrkw={}, name='Output_Directory_Path', library=stdlib)
    masterNet.addNode(Output_Directory_Path_2,572,44)
    Output_Directory_Path_2.inputPortByName['directory'].widget.set(r"TrajQRClustering_0.1_output", run=False)
    apply(Output_Directory_Path_2.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named Output_Directory_Path in network masterNet"
    print_exc()
    Output_Directory_Path_2=None

try:
    ## saving node TrajQR2 ##
    from Adt.Macro.TrajQR import TrajQR
    TrajQR2_3 = TrajQR(constrkw={}, name='TrajQR2', library=Adt)
    masterNet.addNode(TrajQR2_3,297,173)
    apply(TrajQR2_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    TrajQR_kryptonite_nbcr_net_7 = TrajQR2_3.macroNetwork.nodes[3]
    TrajQR_kryptonite_nbcr_net_7.inputPortByName['license'].widget.set(0, run=False)
    TrajQR_kryptonite_nbcr_net_7.inputPortByName['qh_filename'].widget.set(r"qh.tre", run=False)
    TrajQR_kryptonite_nbcr_net_7.inputPortByName['qr_filename'].widget.set(r"qr.out", run=False)
    TrajQR_kryptonite_nbcr_net_7.inputPortByName['localRun'].widget.set(0, run=False)
    TrajQR_kryptonite_nbcr_net_7.inputPortByName['execPath'].widget.set(r"", run=False)
    SelectOnExtension_8 = TrajQR2_3.macroNetwork.nodes[4]
    SelectOnExtension_8.inputPortByName['extension'].widget.set(r".out", run=False)
    Index_9 = TrajQR2_3.macroNetwork.nodes[5]
    apply(Index_9.inputPortByName['index'].widget.configure, (), {'max': 0, 'min': -1})
    Index_9.inputPortByName['index'].widget.set(0, run=False)

    ## saving connections for network TrajQR2 ##
    TrajQR2_3.macroNetwork.freeze()
    TrajQR2_3.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_4 = TrajQR2_3.macroNetwork.ipNode
    input_Ports_4.outputPorts[1].configure(name='Make_Zip_File_input_directory')
    input_Ports_4.outputPorts[2].configure(name='TrajQR_kryptonite_nbcr_net_rmsd')
    input_Ports_4.outputPorts[3].configure(name='GetSelectedPDBs_pdb_out_dir')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_5 = TrajQR2_3.macroNetwork.opNode
    output_Ports_5.inputPorts[1].configure(singleConnection='auto')
    output_Ports_5.inputPorts[1].configure(name='GetSelectedPDBs_pdb_out_dir')
    TrajQR2_3.inputPorts[0].configure(name='Make_Zip_File_input_directory')
    TrajQR2_3.inputPorts[0].configure(datatype='string')
    TrajQR2_3.inputPorts[1].configure(name='TrajQR_kryptonite_nbcr_net_rmsd')
    TrajQR2_3.inputPorts[1].configure(datatype='string')
    TrajQR2_3.inputPorts[2].configure(name='GetSelectedPDBs_pdb_out_dir')
    TrajQR2_3.inputPorts[2].configure(datatype='string')
    ## configure MacroNode input ports
    TrajQR2_3.outputPorts[0].configure(name='GetSelectedPDBs_pdb_out_dir')
    TrajQR2_3.outputPorts[0].configure(datatype='string')
    ## configure MacroNode output ports
    TrajQR2_3.shrink()
    apply(TrajQR2_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore TrajQR named TrajQR2 in network masterNet"
    print_exc()
    TrajQR2_3=None

try:
    ## saving node DPFTemplateBrowser ##
    from Adt.Input.DPFTemplateBrowser import DPFTemplateBrowser
    DPFTemplateBrowser_11 = DPFTemplateBrowser(constrkw={}, name='DPFTemplateBrowser', library=Adt)
    masterNet.addNode(DPFTemplateBrowser_11,823,165)
    DPFTemplateBrowser_11.inputPortByName['dpf_template_file'].widget.set(r"TrajQRClustering_0.1_input/GpfDpf/2HTY_A.dpf", run=False)
    apply(DPFTemplateBrowser_11.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DPFTemplateBrowser named DPFTemplateBrowser in network masterNet"
    print_exc()
    DPFTemplateBrowser_11=None

try:
    ## saving node GPFTemplateBrowser ##
    from Adt.Input.GPFTemplateBrowser import GPFTemplateBrowser
    GPFTemplateBrowser_12 = GPFTemplateBrowser(constrkw={}, name='GPFTemplateBrowser', library=Adt)
    masterNet.addNode(GPFTemplateBrowser_12,411,161)
    GPFTemplateBrowser_12.inputPortByName['gpf_template_file'].widget.set(r"TrajQRClustering_0.1_input/GpfDpf/2HTY_A.gpf", run=False)
    apply(GPFTemplateBrowser_12.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore GPFTemplateBrowser named GPFTemplateBrowser in network masterNet"
    print_exc()
    GPFTemplateBrowser_12=None

try:
    ## saving node MakeGpfDpfCopies ##
    from Adt.Input.MakeGpfDpfCopies import MakeGpfDpfCopies
    MakeGpfDpfCopies_13 = MakeGpfDpfCopies(constrkw={}, name='MakeGpfDpfCopies', library=Adt)
    masterNet.addNode(MakeGpfDpfCopies_13,502,400)
    apply(MakeGpfDpfCopies_13.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore MakeGpfDpfCopies named MakeGpfDpfCopies in network masterNet"
    print_exc()
    MakeGpfDpfCopies_13=None

#masterNet.run()
masterNet.freeze()

## saving connections for network TrajQRClustering-0.1 ##
if RMSD_1 is not None and TrajQR2_3 is not None:
    try:
        masterNet.connectNodes(
            RMSD_1, TrajQR2_3, "string", "TrajQR_kryptonite_nbcr_net_rmsd", blocking=True
            , splitratio=[0.34136199777603804, 0.27589115652596063])
    except:
        print "WARNING: failed to restore connection between RMSD_1 and TrajQR2_3 in network masterNet"
if Output_Directory_Path_2 is not None and TrajQR2_3 is not None:
    try:
        masterNet.connectNodes(
            Output_Directory_Path_2, TrajQR2_3, "directory", "GetSelectedPDBs_pdb_out_dir", blocking=True
            , splitratio=[0.48508950256597771, 0.43046378270767627])
    except:
        print "WARNING: failed to restore connection between Output_Directory_Path_2 and TrajQR2_3 in network masterNet"
if GPFTemplateBrowser_12 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            GPFTemplateBrowser_12, MakeGpfDpfCopies_13, "gpf_template", "gpf_file", blocking=True
            , splitratio=[0.28822475891698995, 0.59826911412993689])
    except:
        print "WARNING: failed to restore connection between GPFTemplateBrowser_12 and MakeGpfDpfCopies_13 in network masterNet"
if DPFTemplateBrowser_11 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            DPFTemplateBrowser_11, MakeGpfDpfCopies_13, "dpf_template", "dpf_file", blocking=True
            , splitratio=[0.35483561885882675, 0.39404164598930946])
    except:
        print "WARNING: failed to restore connection between DPFTemplateBrowser_11 and MakeGpfDpfCopies_13 in network masterNet"
if TrajQR2_3 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            TrajQR2_3, MakeGpfDpfCopies_13, "GetSelectedPDBs_pdb_out_dir", "struct_dir", blocking=True
            , splitratio=[0.42819979143655329, 0.42037477696550413])
    except:
        print "WARNING: failed to restore connection between TrajQR2_3 and MakeGpfDpfCopies_13 in network masterNet"
if Directory_Containing_Input_PDB_Files_0 is not None and TrajQR2_3 is not None:
    try:
        masterNet.connectNodes(
            Directory_Containing_Input_PDB_Files_0, TrajQR2_3, "AbsPath_directory", "Make_Zip_File_input_directory", blocking=True
            , splitratio=[0.47204038642969631, 0.30096756980904804])
    except:
        print "WARNING: failed to restore connection between Directory_Containing_Input_PDB_Files_0 and TrajQR2_3 in network masterNet"
masterNet.runOnNewData.value = False

if __name__=='__main__':
    from sys import argv
    lNodePortValues = []
    if (len(argv) > 1) and argv[1].startswith('-'):
        lArgIndex = 2
    else:
        lArgIndex = 1
    while lArgIndex < len(argv) and argv[lArgIndex][-3:]!='.py':
        lNodePortValues.append(argv[lArgIndex])
        lArgIndex += 1
    masterNet.setNodePortValues(lNodePortValues)
    if '--help' in argv or '-h' in argv: # show help
        masterNet.helpForNetworkAsApplication()
    elif '-w' in argv: # run without Vision and exit
         # create communicator
        from NetworkEditor.net import Communicator
        masterNet.communicator = Communicator(masterNet)
        print 'Communicator listening on port:', masterNet.communicator.port

        import socket
        f = open(argv[0]+'.sock', 'w')
        f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
                         masterNet.communicator.port))
        f.close()

        masterNet.run()

    else: # stand alone application while vision is hidden
        if '-e' in argv: # run and exit
            masterNet.run()
        elif '-r' in argv or len(masterNet.userPanels) == 0: # no user panel => run
            masterNet.run()
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)
        else: # user panel
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)





import TkTreectrl, ImageTk, os, Tkinter, weakref
#from ImageTk import PhotoImage
import tkFont
from raccoonGUI_PrjManagerTree import Node, Research, Project, Experiment, VirtualScreening




class ResultsTree:
    """ build a tree based on the filesystem structure
        project/exp/vs

        provides a mechanism for selection of items
    """

    def __init__(self, parent, path, name = None):

        self.parent = parent
        self.path = path
        self._cosmeticInit()
        self._initTree()
        self.buildTreeGraph()
        self.name = name
        Tkinter.wantobjects=0


        
    def _cosmeticInit(self):
        #self._iconpath="icons/"
        from CADD.Raccoon import ICONPATH
        from CADD.Raccoon import RESULTFILE
        self.RESULTFILE = RESULTFILE
        self._iconpath = ICONPATH

        family = 'Bitstream vera sans'
        family = 'Arial'

        helvisB=tkFont.Font(family=family,size=9,weight="bold")
        helvis=tkFont.Font(family=family,size=9)
        #helvis=tkFont.Font(family="Helvetica",size=9)
        helvis_underline=tkFont.Font(family=family,size=9, underline=1)
        small_font=tkFont.Font(family=family,size=8)
        helvisB10=tkFont.Font(family=family,size=10,weight='bold')
        #courier=tkFont.Font(family='Courier', size = courier_size)

        self.FONTbold=helvisB
        self.FONT=helvis
        self.FONTmini = tkFont.Font(family=family,size=8)
        self.FONTnano = tkFont.Font(family=family,size=1)
        self.FONTmed = tkFont.Font(family=family,size=10, weight='bold')
        self.FONTboldTitle=helvisB10
        self.FONTsmall = small_font
        self.FONTunderline = helvis_underline



        

    def _initTree(self):
        #print "INIT TREE"
        self.treeObj = TkTreectrl.ScrolledTreectrl(self.parent)
        self.treeObj.pack(expand=1,fill='both', padx=5,pady=3)
        self.tree = self.treeObj.treectrl

        self.itemToVs = {}
        # checkbox 
        self.tree.state_define('Checked')
        
        self._checkedIcon = check = \
            Tkinter.PhotoImage(master=self.parent,
            data=('R0lGODlhDQANABEAACwAAAAADQANAIEAAAB/f3/f39',
                   '////8CJ4yPNgHtLxYYtNbIbJ146jZ0gzeCIuhQ53N',
                   'JVNpmryZqsYDnemT3BQA7'))
        self._unCheckedIcon = uncheck = \
            Tkinter.PhotoImage(master=self.parent,
            data=('R0lGODlhDQANABEAACwAAAAADQANAI',
            'EAAAB/f3/f39////8CIYyPNgHtLxYYtNbIrMZTX+l9WThwZAmSppqGmADHcnRaBQA7'))

        el_image = self.tree.element_create(type='image', image=(
            check, 'Checked', uncheck, ()))
        self.checkboxStyle = styleCheckbox = self.tree.style_create()
        self.tree.style_elements(styleCheckbox, el_image)

        self.tree.style_layout(styleCheckbox, el_image, padx=9, pady=2)

        self.research = Research()
        self.tree.itemToObj={ TkTreectrl.ROOT : self.research } # gives obj from tree items
        self.tree.objToItem = {self.research : TkTreectrl.ROOT} # give tree items from obj

        self.tree.columns = []
        firstcol = self.tree.column_create(text='Results in path')
        self.tree.columns.append(firstcol)
        self.tree.configure(treecolumn=firstcol, showbuttons=1, font=self.FONT)
        self.tree.columns.append(self.tree.column_create(text='Selected', expand=1))

        self._molIcon = molIcon = ImageTk.PhotoImage( file=os.path.join(self._iconpath, 'water2.png'))
        self._treeStyles = {}

        # project/exp style
        prj_style = self.tree.style_create()
        p_image = self.tree.element_create(type=TkTreectrl.IMAGE,
                                    image=(molIcon, TkTreectrl.OPEN, molIcon, ''))
        p_text = self.tree.element_create(type=TkTreectrl.TEXT,  fill=('white', TkTreectrl.SELECTED))
        p_select = self.tree.element_create(type=TkTreectrl.RECT, showfocus=1,
                                     fill=('blue4', TkTreectrl.SELECTED))
                
        self.tree.style_elements(prj_style, p_image, p_select, p_text)
        self.tree.style_layout(prj_style, p_image, pady=1)
        self.tree.style_layout(prj_style, p_select, union=(p_text,), padx=1, pady=1, squeeze='')
        self.tree.style_layout(prj_style, p_text, padx=1, pady=1, ipadx=2,ipady=2) #, squeeze='y')
        self.tree.itemstyle_set(TkTreectrl.ROOT, self.tree.columns[0], prj_style)
        
        self._treeStyles['prj'] = { 'img' : p_image, 'txt' : p_text, 'style' : prj_style}

        # style for jobs
        job_style = self.tree.style_create()
        job_image = self.tree.element_create(type=TkTreectrl.IMAGE,
                                    image=(molIcon, TkTreectrl.OPEN, molIcon, ''))
        self.tree.style_elements(job_style, job_image, p_select)
        self.tree.style_layout(job_style, job_image, pady=2)
        self.tree.style_layout(job_style, p_select, union=(job_image,), padx=1, pady=1, ipadx=1,ipady=1 ) #, squeeze='y')
        self._treeStyles['job'] = { 'img' : job_image, 'style' : job_style}

        
        # style for string cell
        styleString = self.tree.style_create()
        cel_text = self.tree.element_create(type=TkTreectrl.TEXT,  fill=('white', TkTreectrl.SELECTED))
        self.tree.style_elements(styleString, p_select, cel_text)
        self.tree.style_layout(styleString, cel_text, padx=1, pady=1, squeeze='y')
        self.tree.style_layout(styleString, p_select, union=(cel_text,), padx=1, pady=1, ipadx=1,ipady=1 )
        self._treeStyles['cell'] = { 'txt' : cel_text, 'style' : styleString}


        # bindings
        self.tree.notify_bind('<Expand-before>', self.expandNode_cb)
        self.tree.bind('<1>', self.on_button1)


    def expandNode_cb(self, event):
        #print "CALLED EXPANDNODE", event, event.item
        item = event.item
        obj = self.tree.itemToObj.get(item, None)
        item = self.tree.objToItem.get(obj, None)
        if item is None:
            raise RuntimeError ('expanding node without item')
    
        if self.tree.item_numchildren(item)==len(obj.children):
            # directory has already been drawn, don't bother to track changes to the directory structure here
            return

        if len(obj.children):
            self.tree.item_config(item, button=True)

        prj = self._treeStyles['prj']
        p_txt = prj['txt']
        p_style = prj['style']

        for child in obj.children:
            if self.tree.objToItem.has_key(child):
                continue
            button = len(child.children) > 0
            new = self.tree.create_item(parent=item, button=button, open=0)[0]
            # create bi-directional lookup
            self.tree.itemToObj[new] = child
            self.tree.objToItem[child] = new 
            self.tree.itemstyle_set(new, self.tree.columns[0], p_style)
            self.tree.itemstyle_set(new, self.tree.columns[1], self.checkboxStyle)
            self.tree.itemelement_config(new, self.tree.columns[0], p_txt, text=child.name, font=self.FONTbold)

            t =  self.tree
            # copy state from parent
            state = t.itemstate_forcolumn(item, t.columns[1])
            if state is not None:
                t.itemstate_forcolumn(new, t.columns[1], state)
                if isinstance(obj, VirtualScreening):
                    self.itemToObj[new] = child

    def on_button1(self, event):
        t = event.widget
        identify = t.identify(event.x, event.y)
        
        if identify:
            try:
                item, column, element = identify[1], identify[3], identify[5]
                if int(column)==1: # The value column was pressed

                    ## handle button click on check buttons in columns 1 (i.e. values)
   
                    t.itemstate_forcolumn(item, column, '~Checked')
                    state = t.itemstate_forcolumn(item, t.columns[1])
                    obj = t.itemToObj[item]
                    self.propagateCheckButtonDown( t, obj, state)
                    if state == None: # if uncheck, uncheck all ancestors
                        for parent in t.item_ancestors(item):
                            t.itemstate_forcolumn(parent, 1, '!Checked')
                            if t.item_ancestors(parent):
                                t.itemToObj[parent].selected = False
                    else:
                        for parent in t.item_ancestors(item):
                            if t.item_ancestors(parent):
                                t.itemToObj[parent].selected = True
                    

                    #if state is not None:
                    #    t.itemstate_forcolumn(new, t.columns[1], state)
                    return 'break'
            except IndexError:
                pass
 
    def propagateCheckButtonDown(self, tree, obj, state):
        # go down the tree and check all buttons
        for child in obj.children:
            item = tree.objToItem.get(child, None)
            if state == None:
                if item:    
                    tree.itemstate_forcolumn(item, 1, '!Checked')
                child.selected = False

            else:
                if item:
                    tree.itemstate_forcolumn(item, 1, 'Checked')
                child.selected = True
                
            self.propagateCheckButtonDown(tree, child, state)





    def buildTreeGraph(self, logname = ''):
        """walk the path and build prj/exp/vs structure

            a dirtree is accepted if the log file is
            found in it
        """
        def _getdir(path):
            dirlist = []
            for d in os.listdir(path):
                dfull = os.path.join(path,d)
                if os.path.isdir(dfull):
                    dirlist.append(dfull)
            return dirlist
            #return  [ d for d in os.listdir(path) if os.path.isdir(os.path.join(path,d)) ]

        if logname == '':
            logname = self.RESULTFILE

        prj_list = _getdir(self.path)
        for p in prj_list:
            exp_list = _getdir(p)
            for e in exp_list:
                vs_list = _getdir(e)
                for v in vs_list:
                    if os.path.isfile(os.path.join(v, logname)):
                        # project
                        pname = os.path.basename(p)
                        prj_obj = self.research.getChild(pname)
                        if not prj_obj:
                            prj_obj = Project(pname)
                            prj_obj.selected = False
                            self.research.addChild(prj_obj)
                        # experiment
                        ename = os.path.basename(e)
                        exp_obj = prj_obj.getChild(ename)
                        if not exp_obj:
                            exp_obj = Experiment(ename)
                            exp_obj.selected = False
                            prj_obj.addChild(exp_obj)
                        # vs
                        vname = os.path.basename(v)
                        vs_obj = VirtualScreening(vname, os.path.getmtime(v), None, None, None,None)
                        vs_obj.path = os.path.join( self.path, p, e, vname )
                        vs_obj.selected = False
                        exp_obj.addChild(vs_obj)
        self.status = len(self.research.children) > 0


        
        






#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#



class MultiThread:



#!/bin/ksh ~/.mgltools/pythonsh
########################################################################
#
#    Vision Network - Python source code - file generated by vision
#    Tuesday 15 February 2011 10:24:37 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/CADD/workflows/visualization/VSResults_0.1_pmvnet.py,v 1.1 2011/04/15 16:21:41 nadya Exp $
#
# $Id: VSResults_0.1_pmvnet.py,v 1.1 2011/04/15 16:21:41 nadya Exp $
#


if __name__=='__main__':
    from sys import argv
    if '--help' in argv or '-h' in argv or '-w' in argv: # run without Vision
        withoutVision = True
        from Vision.VPE import NoGuiExec
        ed = NoGuiExec()
        from NetworkEditor.net import Network
        import os
        masterNet = Network("process-"+str(os.getpid()))
        ed.addNetwork(masterNet)
    else: # run as a stand alone application while vision is hidden
        withoutVision = False
        from Vision import launchVisionToRunNetworkAsApplication, mainLoopVisionToRunNetworkAsApplication
	if '-noSplash' in argv:
	    splash = False
	else:
	    splash = True
        masterNet = launchVisionToRunNetworkAsApplication(splash=splash)
        import os
        masterNet.filename = os.path.abspath(__file__)
from traceback import print_exc
## loading libraries ##
from Pmv.VisionInterface.PmvNodes import pmvlib
from Vision.StandardNodes import stdlib
try:
    masterNet
except (NameError, AttributeError): # we run the network outside Vision
    from NetworkEditor.net import Network
    masterNet = Network()

masterNet.getEditor().addLibraryInstance(pmvlib,"Pmv.VisionInterface.PmvNodes", "pmvlib")

masterNet.getEditor().addLibraryInstance(stdlib,"Vision.StandardNodes", "stdlib")

try:
    ## saving node GetReceptors ##
    from Vision.StandardNodes import Generic
    GetReceptors_0 = Generic(constrkw={}, name='GetReceptors', library=stdlib)
    masterNet.addNode(GetReceptors_0,12,104)
    apply(GetReceptors_0.addInputPort, (), {'singleConnection': True, 'name': 'directory', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(GetReceptors_0.addOutputPort, (), {'name': 'filelist', 'datatype': 'list', 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan'})
    apply(GetReceptors_0.addOutputPort, (), {'name': 'filelist_paths', 'datatype': 'list', 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan'})
    code = """def doit(self, directory):
    import glob, os
    match_str = "*"
    if directory is not None:
        cwd = os.getcwd()
        os.chdir(directory)
    try:
        pdirs = glob.glob(match_str)
        values = []

        for i in pdirs:
            if os.path.exists(i + os.sep + '''summarize_results.txt'''):
                values.append(i)
            
        filenames = [ os.path.join(directory, x) for x in values ]
        self.outputData(filelist=values, filelist_paths=filenames)
    finally:
        if directory is not None:
            os.chdir(cwd)































## to ouput data on port filelist_paths use
## self.outputData(filelist_paths=data)



















"""
    GetReceptors_0.configure(function=code)
    apply(GetReceptors_0.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore Generic named GetReceptors in network masterNet"
    print_exc()
    GetReceptors_0=None

try:
    ## saving node ReceptorList ##
    from Vision.StandardNodes import ScrolledListNE
    ReceptorList_1 = ScrolledListNE(constrkw={}, name='ReceptorList', library=stdlib)
    masterNet.addNode(ReceptorList_1,13,163)
    apply(ReceptorList_1.inputPortByName['selection'].widget.configure, (), {'choices': ['2HTY_A', '2HU4_B']})
    ReceptorList_1.inputPortByName['selection'].widget.set(r"", run=False)
    code = """def doit(self, valueList, nameList, selection):
    widget = self.inputPorts[2].widget

    # do we run the first time, and have been restored from a network? (then
    # we have both valueList and selection specified)

    if self.inputPorts[0].hasNewValidData():
        selection = None
    
    if valueList and selection and self.firstRun is True:
        self.firstRun = False
        self.objectList = valueList
        self.nameList = nameList
        data = self.getData(selection, nameList)
        if data is not None:
            self.outputData(selection=data)
        return

    # only rebuild widget if we receive a new valueList
    if self.inputPorts[0].hasNewValidData() or self.inputPorts[1].hasNewValidData():
        if valueList != self.objectList or self.nameList != nameList:
            self.objectList = valueList
            self.nameList = nameList
            if nameList:
                vList = nameList
            else:
                vList = []
                for data in valueList:
                    if type(data) == types.StringType:
                        vList.append(data)
                    else:
                        vList.append(repr(data))
            if widget is not None:
                widget.setlist(vList)
                widget.configure(choices=vList)
                widget.widget.setentry('''''')
            return

    if selection:
        data = self.getData(selection, nameList)
        if data is not None:
            self.outputData(selection=data)

"""
    ReceptorList_1.configure(function=code)
    apply(ReceptorList_1.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore ScrolledListNE named ReceptorList in network masterNet"
    print_exc()
    ReceptorList_1=None

try:
    ## saving node GetLigands ##
    from Vision.StandardNodes import Generic
    GetLigands_2 = Generic(constrkw={}, name='GetLigands', library=stdlib)
    masterNet.addNode(GetLigands_2,245,106)
    apply(GetLigands_2.addInputPort, (), {'singleConnection': True, 'name': 'receptor', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(GetLigands_2.addInputPort, (), {'singleConnection': True, 'name': 'receptorlist', 'cast': True, 'datatype': 'list', 'defaultValue': None, 'required': True, 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan', 'originalDatatype': 'None'})
    apply(GetLigands_2.addOutputPort, (), {'name': 'ligandlist', 'datatype': 'list', 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan'})
    apply(GetLigands_2.addOutputPort, (), {'name': 'ligandlist_paths', 'datatype': 'list', 'height': 12, 'width': 12, 'shape': 'oval', 'color': 'cyan'})
    apply(GetLigands_2.addOutputPort, (), {'name': 'liganddir', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
    code = """def doit(self, receptor, receptorlist):
        ligandlist = []
        ligandlist_paths = []
        
        for i in receptorlist:
            if os.path.basename(i) == receptor:
                liganddir = i
                break

        lf = liganddir + os.sep + '''summarize_results.txt'''

        f = open(lf)
        f.readline()

        for i in f.readlines():
            ligand = os.path.basename(str(i).split(''',''')[0])
            ligandlist.append(ligand)
            ligandlist_paths.append(liganddir + os.sep + ligand)
        
        f.close()
            
	pass
## to ouput data on port ligandlist use
        self.outputData(ligandlist=ligandlist, ligandlist_paths=ligandlist_paths, liganddir=liganddir)
## to ouput data on port ligandlist_paths use
## self.outputData(ligandlist_paths=data)
## to ouput data on port liganddir use
## self.outputData(liganddir=data)




















"""
    GetLigands_2.configure(function=code)
    apply(GetLigands_2.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore Generic named GetLigands in network masterNet"
    print_exc()
    GetLigands_2=None

try:
    ## saving node LoadMolecule5 ##
    from NetworkEditor.macros import MacroNode
    LoadMolecule5_3 = MacroNode(name='LoadMolecule5')
    masterNet.addNode(LoadMolecule5_3, 451, 480)
    output_Ports_5 = LoadMolecule5_3.macroNetwork.opNode
    apply(output_Ports_5.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    output_Ports_5.move(235, 451)
    from Vision.StandardNodes import Eval
    abs_path_6 = Eval(constrkw={}, name='abs_path', library=stdlib)
    LoadMolecule5_3.macroNetwork.addNode(abs_path_6,260,76)
    apply(abs_path_6.inputPortByName['in1'].configure, (), {'datatype': 'string', 'defaultValue': None, 'required': True, 'originalDatatype': 'None'})
    apply(abs_path_6.outputPortByName['result'].configure, (), {'datatype': 'string'})
    abs_path_6.inputPortByName['command'].widget.set(r"os.path.abspath(in1)", run=False)
    abs_path_6.inputPortByName['importString'].widget.set(r"", run=False)
    apply(abs_path_6.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Vision.StandardNodes import Generic
    FindMolPath_7 = Generic(constrkw={}, name='FindMolPath', library=stdlib)
    LoadMolecule5_3.macroNetwork.addNode(FindMolPath_7,217,149)
    apply(FindMolPath_7.addInputPort, (), {'singleConnection': True, 'name': 'molname', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(FindMolPath_7.addInputPort, (), {'singleConnection': True, 'name': 'directory', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(FindMolPath_7.addOutputPort, (), {'name': 'molpath', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
    code = """def doit(self, molname, directory):
        for d in os.listdir(directory):
            da = os.path.join(directory, d)
            if os.path.isdir(da):
                pdbqt = da + os.sep + molname + '''.pdbqt'''
                pdb = da + os.sep + molname + '''.pdb'''
        
                if os.path.exists(pdbqt):
                    self.outputData(molpath=pdbqt)
                elif os.path.exists(pdb):
                    self.outputData(molpath=pdb)
                else:
                    self.outputData(molpath="")
## to ouput data on port molpath use
#        self.outputData(molpath="")































"""
    FindMolPath_7.configure(function=code)
    apply(FindMolPath_7.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_8 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(Pmv_8,101,189)
    Pmv_8.inputPortByName['cmdName'].widget.set(r"readMolecule", run=False)
    Pmv_8.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_8.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvRunCommand
    readMolecule_9 = PmvRunCommand(functionOrString='Run_Command', command=masterNet.editor.vf.readMolecule, namedArgs={'parser': None}, posArgsNames=['filename'], constrkw={'functionOrString': "'Run_Command'", 'command': 'masterNet.editor.vf.readMolecule', 'namedArgs': {'parser': None}, 'posArgsNames': ['filename']}, name='readMolecule', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(readMolecule_9,199,248)
    apply(readMolecule_9.inputPortByName['filename'].configure, (), {'datatype': 'string', 'defaultValue': None, 'originalDatatype': 'None'})
    apply(readMolecule_9.inputPortByName['parser'].configure, (), {'defaultValue': None})
    apply(readMolecule_9.configure, (), {'specialPortsVisible': True})
    readMolecule_9.inputPortByName['importString'].widget.set(r"", run=False)
    apply(readMolecule_9.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_10 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(Pmv_10,101,293)
    Pmv_10.inputPortByName['cmdName'].widget.set(r"computeMSMS", run=False)
    Pmv_10.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_10.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvRunCommand
    computeMSMS_11 = PmvRunCommand(functionOrString='Run_Command', command=masterNet.editor.vf.computeMSMS, namedArgs={'hdensity': 6.0, 'hdset': 'None', 'surfName': 'MSMS-MOL', 'density': 3.0, 'perMol': True, 'noHetatm': False, 'display': True, 'pRadius': 1.5}, posArgsNames=['nodes'], constrkw={'functionOrString': "'Run_Command'", 'command': 'masterNet.editor.vf.computeMSMS', 'namedArgs': {'hdensity': 6.0, 'hdset': 'None', 'surfName': 'MSMS-MOL', 'density': 3.0, 'perMol': True, 'noHetatm': False, 'display': True, 'pRadius': 1.5}, 'posArgsNames': ['nodes']}, name='computeMSMS', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(computeMSMS_11,200,352)
    apply(computeMSMS_11.inputPortByName['nodes'].configure, (), {'datatype': 'string', 'defaultValue': None, 'originalDatatype': 'None'})
    apply(computeMSMS_11.inputPortByName['hdensity'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['hdset'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['surfName'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['density'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['perMol'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['display'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.inputPortByName['pRadius'].configure, (), {'defaultValue': None})
    apply(computeMSMS_11.configure, (), {'specialPortsVisible': True})
    computeMSMS_11.inputPortByName['importString'].widget.set(r"", run=False)
    computeMSMS_11.inputPortByName['hdensity'].widget.set(6.0, run=False)
    computeMSMS_11.inputPortByName['hdset'].widget.set(r"None", run=False)
    computeMSMS_11.inputPortByName['surfName'].widget.set(r"MSMS-MOL", run=False)
    computeMSMS_11.inputPortByName['density'].widget.set(3.0, run=False)
    computeMSMS_11.inputPortByName['perMol'].widget.set(1, run=False)
    computeMSMS_11.inputPortByName['noHetatm'].widget.set(0, run=False)
    computeMSMS_11.inputPortByName['display'].widget.set(1, run=False)
    computeMSMS_11.inputPortByName['pRadius'].widget.set(1.5, run=False)
    apply(computeMSMS_11.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1})
    from Pmv.VisionInterface.PmvNodes import PmvRunCommand
    deleteAllMolecules_12 = PmvRunCommand(functionOrString='Run_Command', command=masterNet.editor.vf.deleteAllMolecules, constrkw={'functionOrString': "'Run_Command'", 'command': 'masterNet.editor.vf.deleteAllMolecules'}, name='deleteAllMolecules', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(deleteAllMolecules_12,358,153)
    apply(deleteAllMolecules_12.configure, (), {'specialPortsVisible': True})
    deleteAllMolecules_12.inputPortByName['importString'].widget.set(r"", run=False)
    apply(deleteAllMolecules_12.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_13 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    LoadMolecule5_3.macroNetwork.addNode(Pmv_13,387,60)
    Pmv_13.inputPortByName['cmdName'].widget.set(r"deleteAllMolecules", run=False)
    Pmv_13.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_13.configure, (), {'paramPanelImmediate': 1, 'expanded': False})

    ## saving connections for network LoadMolecule5 ##
    LoadMolecule5_3.macroNetwork.freeze()
    input_Ports_4 = LoadMolecule5_3.macroNetwork.ipNode
    if input_Ports_4 is not None and FindMolPath_7 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                input_Ports_4, FindMolPath_7, "new", "molname", blocking=True
                , splitratio=[0.67694924181626903, 0.22784911266549582])
        except:
            print "WARNING: failed to restore connection between input_Ports_4 and FindMolPath_7 in network LoadMolecule5_3.macroNetwork"
    if input_Ports_4 is not None and abs_path_6 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                input_Ports_4, abs_path_6, "new", "in1", blocking=True
                , splitratio=[0.74684717272792378, 0.70618943139915813])
        except:
            print "WARNING: failed to restore connection between input_Ports_4 and abs_path_6 in network LoadMolecule5_3.macroNetwork"
    if abs_path_6 is not None and FindMolPath_7 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                abs_path_6, FindMolPath_7, "result", "directory", blocking=True
                , splitratio=[0.74896498645804965, 0.31322600506795284])
        except:
            print "WARNING: failed to restore connection between abs_path_6 and FindMolPath_7 in network LoadMolecule5_3.macroNetwork"
    if Pmv_8 is not None and readMolecule_9 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                Pmv_8, readMolecule_9, "cmd", "command", blocking=True
                , splitratio=[0.20198223086616679, 0.29898362394376937])
        except:
            print "WARNING: failed to restore connection between Pmv_8 and readMolecule_9 in network LoadMolecule5_3.macroNetwork"
    if FindMolPath_7 is not None and readMolecule_9 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                FindMolPath_7, readMolecule_9, "molpath", "filename", blocking=True
                , splitratio=[0.21636244059584375, 0.72623026114617018])
        except:
            print "WARNING: failed to restore connection between FindMolPath_7 and readMolecule_9 in network LoadMolecule5_3.macroNetwork"
    if Pmv_10 is not None and computeMSMS_11 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                Pmv_10, computeMSMS_11, "cmd", "command", blocking=True
                , splitratio=[0.51805666489740432, 0.65872140393229073])
        except:
            print "WARNING: failed to restore connection between Pmv_10 and computeMSMS_11 in network LoadMolecule5_3.macroNetwork"
    if input_Ports_4 is not None and computeMSMS_11 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                input_Ports_4, computeMSMS_11, "FindMolPath_molname", "nodes", blocking=True
                , splitratio=[0.52821761384014243, 0.62344925894625258])
        except:
            print "WARNING: failed to restore connection between input_Ports_4 and computeMSMS_11 in network LoadMolecule5_3.macroNetwork"
    if readMolecule_9 is not None and computeMSMS_11 is not None:
        try:
            LoadMolecule5_3.macroNetwork.specialConnectNodes(
                readMolecule_9, computeMSMS_11, "trigger", "runNode", blocking=True
                , splitratio=[0.53056199344731425, 0.35524187431681792])
        except:
            print "WARNING: failed to restore connection between readMolecule_9 and computeMSMS_11 in network LoadMolecule5_3.macroNetwork"
    if Pmv_13 is not None and deleteAllMolecules_12 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                Pmv_13, deleteAllMolecules_12, "cmd", "command", blocking=True
                , splitratio=[0.66024693919479915, 0.62418992981442756])
        except:
            print "WARNING: failed to restore connection between Pmv_13 and deleteAllMolecules_12 in network LoadMolecule5_3.macroNetwork"
    if deleteAllMolecules_12 is not None and readMolecule_9 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                deleteAllMolecules_12, readMolecule_9, "result", "parser", blocking=True
                , splitratio=[0.32991431199869631, 0.4313515641191642])
        except:
            print "WARNING: failed to restore connection between deleteAllMolecules_12 and readMolecule_9 in network LoadMolecule5_3.macroNetwork"
    output_Ports_5 = LoadMolecule5_3.macroNetwork.opNode
    if FindMolPath_7 is not None and output_Ports_5 is not None:
        try:
            LoadMolecule5_3.macroNetwork.connectNodes(
                FindMolPath_7, output_Ports_5, "molpath", "new", blocking=True
                , splitratio=[0.59721511886821532, 0.46900469791423793])
        except:
            print "WARNING: failed to restore connection between FindMolPath_7 and output_Ports_5 in network LoadMolecule5_3.macroNetwork"
    LoadMolecule5_3.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_4.outputPorts[1].configure(name='FindMolPath_molname')
    input_Ports_4.outputPorts[2].configure(name='abs_path_in1')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_5.inputPorts[1].configure(singleConnection='auto')
    output_Ports_5.inputPorts[1].configure(name='FindMolPath_molpath')
    LoadMolecule5_3.inputPorts[0].configure(name='FindMolPath_molname')
    LoadMolecule5_3.inputPorts[0].configure(datatype='string')
    LoadMolecule5_3.inputPorts[1].configure(name='abs_path_in1')
    LoadMolecule5_3.inputPorts[1].configure(datatype='string')
    ## configure MacroNode input ports
    LoadMolecule5_3.outputPorts[0].configure(name='FindMolPath_molpath')
    LoadMolecule5_3.outputPorts[0].configure(datatype='string')
    ## configure MacroNode output ports
    LoadMolecule5_3.shrink()
    apply(LoadMolecule5_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore MacroNode named LoadMolecule5 in network masterNet"
    print_exc()
    LoadMolecule5_3=None

try:
    ## saving node ShowConformations6 ##
    from NetworkEditor.macros import MacroNode
    ShowConformations6_14 = MacroNode(name='ShowConformations6')
    masterNet.addNode(ShowConformations6_14, 543, 541)
    output_Ports_16 = ShowConformations6_14.macroNetwork.opNode
    apply(output_Ports_16.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    output_Ports_16.move(220, 365)
    from Vision.StandardNodes import Generic
    GetLigandDLG_17 = Generic(constrkw={}, name='GetLigandDLG', library=stdlib)
    ShowConformations6_14.macroNetwork.addNode(GetLigandDLG_17,253,127)
    apply(GetLigandDLG_17.addInputPort, (), {'singleConnection': True, 'name': 'ligand_name', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(GetLigandDLG_17.addInputPort, (), {'singleConnection': True, 'name': 'ligand_dir', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(GetLigandDLG_17.addOutputPort, (), {'name': 'ligand_dlg', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
    apply(GetLigandDLG_17.configure, (), {'specialPortsVisible': True})
    code = """def doit(self, ligand_name, ligand_dir):
        ligand_dlg = ligand_dir + os.sep + ligand_name + os.sep + ligand_name + '''.dlg'''

        if not os.path.exists(ligand_dlg):
            ligand_dlg = ligand_dir + os.sep + ligand_name + os.sep + '''outlog.dlg'''

        if not os.path.exists(ligand_dlg):
            print "ERROR: NO ligand dlg found in " + ligand_dir + os.sep + ligand_name + os.sep
            
	pass
## to ouput data on port ligand_dlg use
        self.outputData(ligand_dlg=ligand_dlg)








"""
    GetLigandDLG_17.configure(function=code)
    apply(GetLigandDLG_17.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_18 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    ShowConformations6_14.macroNetwork.addNode(Pmv_18,58,145)
    Pmv_18.inputPortByName['cmdName'].widget.set(r"AD41analyze_readDLG", run=False)
    Pmv_18.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_18.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvRunCommand
    AD41analyze_readDLG_19 = PmvRunCommand(functionOrString='Run_Command', command=masterNet.editor.vf.AD41analyze_readDLG, namedArgs={'ask': 1, 'addToPrevious': 0}, posArgsNames=['dlgFile'], constrkw={'functionOrString': "'Run_Command'", 'command': 'masterNet.editor.vf.AD41analyze_readDLG', 'namedArgs': {'ask': 1, 'addToPrevious': 0}, 'posArgsNames': ['dlgFile']}, name='AD41analyze_readDLG', library=pmvlib)
    ShowConformations6_14.macroNetwork.addNode(AD41analyze_readDLG_19,237,211)
    apply(AD41analyze_readDLG_19.inputPortByName['dlgFile'].configure, (), {'datatype': 'string', 'defaultValue': None, 'originalDatatype': 'None'})
    apply(AD41analyze_readDLG_19.inputPortByName['ask'].configure, (), {'defaultValue': None})
    apply(AD41analyze_readDLG_19.inputPortByName['addToPrevious'].configure, (), {'defaultValue': None})
    apply(AD41analyze_readDLG_19.configure, (), {'specialPortsVisible': True})
    AD41analyze_readDLG_19.inputPortByName['importString'].widget.set(r"", run=False)
    AD41analyze_readDLG_19.inputPortByName['ask'].widget.set(1, run=False)
    AD41analyze_readDLG_19.inputPortByName['addToPrevious'].widget.set(0, run=False)
    apply(AD41analyze_readDLG_19.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_20 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    ShowConformations6_14.macroNetwork.addNode(Pmv_20,59,225)
    Pmv_20.inputPortByName['cmdName'].widget.set(r"AD41analyze_showStates", run=False)
    Pmv_20.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_20.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvRunCommand
    AD41analyze_showStates_21 = PmvRunCommand(functionOrString='Run_Command', command=masterNet.editor.vf.AD41analyze_showStates, namedArgs={'log': 1}, posArgsNames=['mol'], constrkw={'functionOrString': "'Run_Command'", 'command': 'masterNet.editor.vf.AD41analyze_showStates', 'namedArgs': {'log': 1}, 'posArgsNames': ['mol']}, name='AD41analyze_showStates', library=pmvlib)
    ShowConformations6_14.macroNetwork.addNode(AD41analyze_showStates_21,200,285)
    apply(AD41analyze_showStates_21.inputPortByName['mol'].configure, (), {'datatype': 'string', 'defaultValue': None, 'originalDatatype': 'None'})
    apply(AD41analyze_showStates_21.inputPortByName['log'].configure, (), {'defaultValue': None})
    apply(AD41analyze_showStates_21.configure, (), {'specialPortsVisible': True})
    AD41analyze_showStates_21.inputPortByName['importString'].widget.set(r"", run=False)
    AD41analyze_showStates_21.inputPortByName['log'].widget.set(1, run=False)
    apply(AD41analyze_showStates_21.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1})
    from Vision.StandardNodes import Generic
    GetMolList_22 = Generic(constrkw={}, name='GetMolList', library=stdlib)
    ShowConformations6_14.macroNetwork.addNode(GetMolList_22,374,126)
    apply(GetMolList_22.addInputPort, (), {'singleConnection': True, 'name': 'Pmv', 'cast': True, 'datatype': 'PmvInstance', 'defaultValue': None, 'required': True, 'height': 12, 'width': 12, 'shape': 'circle', 'color': '#7A7AFF', 'originalDatatype': 'None'})
    apply(GetMolList_22.addInputPort, (), {'singleConnection': True, 'name': 'in1', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
    apply(GetMolList_22.addInputPort, (), {'singleConnection': True, 'name': 'in2', 'cast': True, 'datatype': 'None', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})
    apply(GetMolList_22.addOutputPort, (), {'name': 'out0', 'datatype': 'None', 'height': 8, 'width': 12, 'shape': 'diamond', 'color': 'white'})
    apply(GetMolList_22.configure, (), {'specialPortsVisible': True})
    code = """def doit(self, Pmv, in1, in2):
        assert Pmv is not None
        receptor = os.path.basename(in2)
        receptor = receptor.split('''.''')[0]

        all = map(lambda x: x.name, Pmv.Mols)
        #count = 0

        #for i in all:
        #    if i != in1 or i != in2:
        #        count = count + 1
        #        all.remove(i)
                #if count == 2:
                #    break

        #Pmv.showMolecules(all, negate=True, log=0)
        
        mols = list()

        for i in all:
            if i != in1 and i != receptor:
                mols.append(i)

        Pmv.showMolecules(mols, negate=True, log=0)
        #Pmv.showMolecules(negate=True, log=0)

	pass
## to ouput data on port out0 use
        self.outputData(out0=mols)

































"""
    GetMolList_22.configure(function=code)
    apply(GetMolList_22.configure, (), {'specialPortsVisible': True, 'paramPanelImmediate': 1, 'expanded': False})
    from Pmv.VisionInterface.PmvNodes import PmvNode
    Pmv_23 = PmvNode(vf=masterNet.editor.vf, constrkw={'vf': 'masterNet.editor.vf'}, name='Pmv', library=pmvlib)
    ShowConformations6_14.macroNetwork.addNode(Pmv_23,375,42)
    apply(Pmv_23.inputPortByName['cmdName'].configure, (), {'defaultValue': None})
    apply(Pmv_23.inputPortByName['molecules'].configure, (), {'defaultValue': None})
    Pmv_23.inputPortByName['cmdName'].widget.set(r"", run=False)
    Pmv_23.inputPortByName['molecules'].widget.set(r"", run=False)
    apply(Pmv_23.configure, (), {'paramPanelImmediate': 1, 'expanded': False})

    ## saving connections for network ShowConformations6 ##
    ShowConformations6_14.macroNetwork.freeze()
    input_Ports_15 = ShowConformations6_14.macroNetwork.ipNode
    if input_Ports_15 is not None and GetLigandDLG_17 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                input_Ports_15, GetLigandDLG_17, "new", "ligand_name", blocking=True
                , splitratio=[0.3098536280134469, 0.48565093299230189])
        except:
            print "WARNING: failed to restore connection between input_Ports_15 and GetLigandDLG_17 in network ShowConformations6_14.macroNetwork"
    if input_Ports_15 is not None and GetLigandDLG_17 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                input_Ports_15, GetLigandDLG_17, "new", "ligand_dir", blocking=True
                , splitratio=[0.33601172059588991, 0.62781207392950156])
        except:
            print "WARNING: failed to restore connection between input_Ports_15 and GetLigandDLG_17 in network ShowConformations6_14.macroNetwork"
    if Pmv_18 is not None and AD41analyze_readDLG_19 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                Pmv_18, AD41analyze_readDLG_19, "cmd", "command", blocking=True
                , splitratio=[0.69124752996188765, 0.74417941624282569])
        except:
            print "WARNING: failed to restore connection between Pmv_18 and AD41analyze_readDLG_19 in network ShowConformations6_14.macroNetwork"
    if GetLigandDLG_17 is not None and AD41analyze_readDLG_19 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                GetLigandDLG_17, AD41analyze_readDLG_19, "ligand_dlg", "dlgFile", blocking=True
                , splitratio=[0.30237922682451068, 0.37504940304702639])
        except:
            print "WARNING: failed to restore connection between GetLigandDLG_17 and AD41analyze_readDLG_19 in network ShowConformations6_14.macroNetwork"
    if Pmv_20 is not None and AD41analyze_showStates_21 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                Pmv_20, AD41analyze_showStates_21, "cmd", "command", blocking=True
                , splitratio=[0.67750223926482356, 0.3916492569990021])
        except:
            print "WARNING: failed to restore connection between Pmv_20 and AD41analyze_showStates_21 in network ShowConformations6_14.macroNetwork"
    if input_Ports_15 is not None and AD41analyze_showStates_21 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                input_Ports_15, AD41analyze_showStates_21, "GetLigandDLG_ligand_name", "mol", blocking=True
                , splitratio=[0.48195714670172196, 0.48045452051969201])
        except:
            print "WARNING: failed to restore connection between input_Ports_15 and AD41analyze_showStates_21 in network ShowConformations6_14.macroNetwork"
    if AD41analyze_readDLG_19 is not None and AD41analyze_showStates_21 is not None:
        try:
            ShowConformations6_14.macroNetwork.specialConnectNodes(
                AD41analyze_readDLG_19, AD41analyze_showStates_21, "trigger", "runNode", blocking=True
                , splitratio=[0.69705190112635451, 0.37248312577637505])
        except:
            print "WARNING: failed to restore connection between AD41analyze_readDLG_19 and AD41analyze_showStates_21 in network ShowConformations6_14.macroNetwork"
    if Pmv_23 is not None and GetMolList_22 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                Pmv_23, GetMolList_22, "PMV", "Pmv", blocking=True
                , splitratio=[0.22281755983671137, 0.33186887241855467])
        except:
            print "WARNING: failed to restore connection between Pmv_23 and GetMolList_22 in network ShowConformations6_14.macroNetwork"
    if input_Ports_15 is not None and GetMolList_22 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                input_Ports_15, GetMolList_22, "GetLigandDLG_ligand_name", "in1", blocking=True
                , splitratio=[0.41038366697012862, 0.62996532422459084])
        except:
            print "WARNING: failed to restore connection between input_Ports_15 and GetMolList_22 in network ShowConformations6_14.macroNetwork"
    if input_Ports_15 is not None and GetMolList_22 is not None:
        try:
            ShowConformations6_14.macroNetwork.connectNodes(
                input_Ports_15, GetMolList_22, "new", "in2", blocking=True
                , splitratio=[0.66011397806266103, 0.33140888638371957])
        except:
            print "WARNING: failed to restore connection between input_Ports_15 and GetMolList_22 in network ShowConformations6_14.macroNetwork"
    ShowConformations6_14.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_15.outputPorts[1].configure(name='GetLigandDLG_ligand_name')
    input_Ports_15.outputPorts[2].configure(name='GetLigandDLG_ligand_dir')
    input_Ports_15.outputPorts[3].configure(name='GetMolList_in2')
    ShowConformations6_14.inputPorts[0].configure(name='GetLigandDLG_ligand_name')
    ShowConformations6_14.inputPorts[0].configure(datatype='string')
    ShowConformations6_14.inputPorts[1].configure(name='GetLigandDLG_ligand_dir')
    ShowConformations6_14.inputPorts[1].configure(datatype='string')
    ShowConformations6_14.inputPorts[2].configure(name='GetMolList_in2')
    ShowConformations6_14.inputPorts[2].configure(datatype='string')
    ## configure MacroNode input ports
    ## configure MacroNode output ports
    ShowConformations6_14.shrink()
    apply(ShowConformations6_14.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore MacroNode named ShowConformations6 in network masterNet"
    print_exc()
    ShowConformations6_14=None

try:
    ## saving node LigandList ##
    from Vision.StandardNodes import ScrolledListNE
    LigandList_24 = ScrolledListNE(constrkw={}, name='LigandList', library=stdlib)
    masterNet.addNode(LigandList_24,246,161)
    apply(LigandList_24.inputPortByName['selection'].widget.configure, (), {'choices': ['diversity0004', 'diversity0005', 'diversity0003', 'diversity0001', 'diversity0002', 'diversity0006']})
    LigandList_24.inputPortByName['selection'].widget.set(r"", run=False)
    code = """def doit(self, valueList, nameList, selection):
    widget = self.inputPorts[2].widget

    # do we run the first time, and have been restored from a network? (then
    # we have both valueList and selection specified)

    if self.inputPorts[0].hasNewValidData():
        selection = None
        
    if valueList and selection and self.firstRun is True:
        self.firstRun = False
        self.objectList = valueList
        self.nameList = nameList
        data = self.getData(selection, nameList)
        if data is not None:
            self.outputData(selection=data)
        return

    # only rebuild widget if we receive a new valueList
    if self.inputPorts[0].hasNewValidData() or self.inputPorts[1].hasNewValidData():
        if valueList != self.objectList or self.nameList != nameList:
            self.objectList = valueList
            self.nameList = nameList
            if nameList:
                vList = nameList
            else:
                vList = []
                for data in valueList:
                    if type(data) == types.StringType:
                        vList.append(data)
                    else:
                        vList.append(repr(data))
            if widget is not None:
                widget.setlist(vList)
                widget.configure(choices=vList)
                widget.widget.setentry('''''')
            return

    if selection:
        data = self.getData(selection, nameList)
        if data is not None:
            self.outputData(selection=data)

"""
    LigandList_24.configure(function=code)
    apply(LigandList_24.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore ScrolledListNE named LigandList in network masterNet"
    print_exc()
    LigandList_24=None

try:
    ## saving node VSOutputDirectory ##
    from Vision.StandardNodes import DirBrowserNE
    VSOutputDirectory_25 = DirBrowserNE(constrkw={}, name='VSOutputDirectory', library=stdlib)
    masterNet.addNode(VSOutputDirectory_25,10,4)
    VSOutputDirectory_25.inputPortByName['directory'].widget.set(r"VSResults_0.1_input", run=False)
    apply(VSOutputDirectory_25.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named VSOutputDirectory in network masterNet"
    print_exc()
    VSOutputDirectory_25=None

#masterNet.run()
masterNet.freeze()

## saving connections for network VSResults-0.1 ##
if GetReceptors_0 is not None and ReceptorList_1 is not None:
    try:
        masterNet.connectNodes(
            GetReceptors_0, ReceptorList_1, "filelist", "valueList", blocking=True
            , splitratio=[0.48905913647612814, 0.29360410336373438])
    except:
        print "WARNING: failed to restore connection between GetReceptors_0 and ReceptorList_1 in network masterNet"
if ReceptorList_1 is not None and GetLigands_2 is not None:
    try:
        masterNet.connectNodes(
            ReceptorList_1, GetLigands_2, "selection", "receptor", blocking=True
            , splitratio=[0.42538364036717291, 0.58358889034815098])
    except:
        print "WARNING: failed to restore connection between ReceptorList_1 and GetLigands_2 in network masterNet"
if GetReceptors_0 is not None and GetLigands_2 is not None:
    try:
        masterNet.connectNodes(
            GetReceptors_0, GetLigands_2, "filelist_paths", "receptorlist", blocking=True
            , splitratio=[0.36424339756927371, 0.30463602511408611])
    except:
        print "WARNING: failed to restore connection between GetReceptors_0 and GetLigands_2 in network masterNet"
LoadMolecule5_3 = masterNet.nodes[3]
ShowConformations6_14 = masterNet.nodes[4]
if LoadMolecule5_3 is not None and ShowConformations6_14 is not None:
    try:
        masterNet.connectNodes(
            LoadMolecule5_3, ShowConformations6_14, "FindMolPath_molpath", "GetMolList_in2", blocking=True
            , splitratio=[0.42667541388043656, 0.68263435741014988])
    except:
        print "WARNING: failed to restore connection between LoadMolecule5_3 and ShowConformations6_14 in network masterNet"
if GetLigands_2 is not None and LigandList_24 is not None:
    try:
        masterNet.connectNodes(
            GetLigands_2, LigandList_24, "ligandlist", "valueList", blocking=True
            , splitratio=[0.43348589067232524, 0.71851748283944694])
    except:
        print "WARNING: failed to restore connection between GetLigands_2 and LigandList_24 in network masterNet"
if ReceptorList_1 is not None and LoadMolecule5_3 is not None:
    try:
        masterNet.connectNodes(
            ReceptorList_1, LoadMolecule5_3, "selection", "FindMolPath_molname", blocking=True
            , splitratio=[0.45144502057638469, 0.25103051955449046])
    except:
        print "WARNING: failed to restore connection between ReceptorList_1 and LoadMolecule5_3 in network masterNet"
if VSOutputDirectory_25 is not None and LoadMolecule5_3 is not None:
    try:
        masterNet.connectNodes(
            VSOutputDirectory_25, LoadMolecule5_3, "AbsPath_directory", "abs_path_in1", blocking=True
            , splitratio=[0.22052704406206425, 0.59085497893808769])
    except:
        print "WARNING: failed to restore connection between VSOutputDirectory_25 and LoadMolecule5_3 in network masterNet"
if LigandList_24 is not None and ShowConformations6_14 is not None:
    try:
        masterNet.connectNodes(
            LigandList_24, ShowConformations6_14, "selection", "GetLigandDLG_ligand_name", blocking=True
            , splitratio=[0.29779314820077629, 0.65106503390095383])
    except:
        print "WARNING: failed to restore connection between LigandList_24 and ShowConformations6_14 in network masterNet"
if GetLigands_2 is not None and ShowConformations6_14 is not None:
    try:
        masterNet.connectNodes(
            GetLigands_2, ShowConformations6_14, "liganddir", "GetLigandDLG_ligand_dir", blocking=True
            , splitratio=[0.56416893580956851, 0.32973982963985227])
    except:
        print "WARNING: failed to restore connection between GetLigands_2 and ShowConformations6_14 in network masterNet"
if VSOutputDirectory_25 is not None and GetReceptors_0 is not None:
    try:
        masterNet.connectNodes(
            VSOutputDirectory_25, GetReceptors_0, "AbsPath_directory", "directory", blocking=True
            , splitratio=[0.41364431314011513, 0.61325277014217927])
    except:
        print "WARNING: failed to restore connection between VSOutputDirectory_25 and GetReceptors_0 in network masterNet"
masterNet.runOnNewData.value = False

if __name__=='__main__':
    from sys import argv
    lNodePortValues = []
    if (len(argv) > 1) and argv[1].startswith('-'):
        lArgIndex = 2
    else:
        lArgIndex = 1
    while lArgIndex < len(argv) and argv[lArgIndex][-3:]!='.py':
        lNodePortValues.append(argv[lArgIndex])
        lArgIndex += 1
    masterNet.setNodePortValues(lNodePortValues)
    if '--help' in argv or '-h' in argv: # show help
        masterNet.helpForNetworkAsApplication()
    elif '-w' in argv: # run without Vision and exit
         # create communicator
        from NetworkEditor.net import Communicator
        masterNet.communicator = Communicator(masterNet)
        print 'Communicator listening on port:', masterNet.communicator.port

        import socket
        f = open(argv[0]+'.sock', 'w')
        f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
                         masterNet.communicator.port))
        f.close()

        masterNet.run()

    else: # stand alone application while vision is hidden
        if '-e' in argv: # run and exit
            masterNet.run()
        elif '-r' in argv or len(masterNet.userPanels) == 0: # no user panel => run
            masterNet.run()
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)
        else: # user panel
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)

